@@ -1,10 +1,10 @@
-/*
- * this is for users how like to sepatet device and button configuration
- * You have to change the table names to:
- *
- * sccpdevices -> sccpdeviceconfig
- * sccplines -> sccpline  
-*/
+--
+-- this is for users how like to sepatet device and button configuration
+-- You have to change the table names to:
+--
+-- sccpdevices -> sccpdeviceconfig
+-- sccplines -> sccpline  
+--
 
 --
 -- Table with line-configuration
@@ -87,24 +87,58 @@ CREATE TABLE IF NOT EXISTS `sccpline` (
 --
 -- Table with button-configuration for device
 --
+-- foreign constrainst:
+--   device -> sccpdevice.name
+--   type -> buttontype enum
+--   name -> if type=='line' then sccpline.name
+--           else free field
+-- unique constraints:
+--   device, instance, type
+--
 CREATE TABLE IF NOT EXISTS `buttonconfig` (
   `device` varchar(15) NOT NULL default '',
   `instance` tinyint(4) NOT NULL default '0',
   `type` enum('line','speeddial','service','feature','empty') NOT NULL default 'line',
   `name` varchar(36) default NULL,
   `options` varchar(100) default NULL,
-  PRIMARY KEY  (`device`,`instance`),
+  PRIMARY KEY  (`device`,`instance`,`type`),
   KEY `device` (`device`),
-  
   FOREIGN KEY (device) REFERENCES sccpdevice(name) ON DELETE CASCADE ON UPDATE CASCADE
 ) ENGINE=INNODB DEFAULT CHARSET=latin1;
 
 
+--
+-- trigger to check buttonconfig sccpline foreign key constrainst:
+--   if type=='line' then check name against sccpline.name column
+--   else free field
+--
+DROP TRIGGER IF EXISTS trg_buttonconfig;
 
+DELIMITER $$
+CREATE TRIGGER trg_buttonconfig BEFORE INSERT ON buttonconfig
+FOR EACH ROW
+BEGIN
+	IF NEW.`type` = 'line' THEN
+		IF (SELECT COUNT(*) FROM `sccpline` WHERE `sccpline`.`name` = NEW.`name`) = 0
+		THEN
+			UPDATE `Foreign key contraint violated: line does not exist in sccpline` SET x=1;
+		END IF;
+	END IF;
+END$$
+DELIMITER ;
 
 --
 -- View for merging device and button configuration
 --
+-- combines sccpdevice and buttonconfig on buttonconfig.device=sccpdevice.name to 
+-- produce a complete chan-sccp-b device entry including multiple buttons seperated by comma's
+--
+-- When altering sccpdevice or buttonconfig, this view needs to be dropped and recreated afterwards
+--
+--
+-- group_concat_max_len = 2048; in my.cnf
+--
+
 CREATE OR REPLACE
 ALGORITHM = MERGE
 VIEW sccpdeviceconfig AS
