@@ -4,10 +4,18 @@
  * \author      Diederik de Groot <ddegroot [at] sf.net>
  * \note        This program is free software and may be modified and distributed under the terms of the GNU Public License.
  *		See the LICENSE file at the top of the source tree.
- *
- * $Date: 2010-11-17 18:10:34 +0100 (Wed, 17 Nov 2010) $
- * $Revision: 2154 $
  */
+#pragma once
+#if defined(__cplusplus) || defined(c_plusplus)
+#  define __BEGIN_EXTERN__ 		\
+extern "C" {
+#  define __END_EXTERN__ }		\
+}
+#else
+#  define __BEGIN_EXTERN__ 
+#  define __END_EXTERN__ 
+#endif
+
 #define ARRAY_LEN(a) (size_t) (sizeof(a) / sizeof(0[a]))
 typedef enum { FALSE = 0, TRUE = 1 } boolean_t;
 
@@ -16,6 +24,10 @@ typedef enum { FALSE = 0, TRUE = 1 } boolean_t;
 #undef offsize
 #define offsize(T, F) 0
 
+#include <sys/types.h>
+#include <stdint.h>
+#include "../../src/sccp_enum.h"
+
 /*
 #define G_OBJ_REF(x) #x,offsize(struct sccp_global_vars,x)
 #define D_OBJ_REF(x) #x,offsize(struct sccp_device,x)
@@ -30,11 +42,6 @@ typedef enum { FALSE = 0, TRUE = 1 } boolean_t;
 #define H_OBJ_REF(x) 0,0
 
 /* dyn config */
-typedef enum {
-	SCCP_CONFIG_NOUPDATENEEDED = 0,
-	SCCP_CONFIG_NEEDDEVICERESET = 1 << 1
-} sccp_configurationchange_t;						/*!< configuration state change */
-
 typedef enum {
 /* *INDENT-OFF* */
 	SCCP_CONFIG_CHANGE_NOCHANGE			= 0,
@@ -65,6 +72,7 @@ enum SCCPConfigOptionType {
 	SCCP_CONFIG_DATATYPE_PARSER			= 1 << 4,
 	SCCP_CONFIG_DATATYPE_STRINGPTR			= 1 << 5,	/* pointer */
 	SCCP_CONFIG_DATATYPE_CHAR			= 1 << 6,
+	SCCP_CONFIG_DATATYPE_ENUM			= 1 << 7,
 /* *INDENT-ON* */
 };
 
@@ -90,15 +98,15 @@ enum SCCPConfigOptionFlag {
  * \brief SCCP Config Option Struct
  */
 typedef struct SCCPConfigOption {
-
 /* *INDENT-ON* */
 	const char *name;							/*!< Configuration Parameter Name */
 	const int offset;
 	const size_t size;
 	enum SCCPConfigOptionType type;						/*!< Data type */
 	const char *generic_parser;
-	const char *str2enumval;
-	const char *enumkeys;
+        sccp_enum_str2intval_t str2intval;
+        sccp_enum_all_entries_t all_entries;
+        const char *parsername;
 	enum SCCPConfigOptionFlag flags;					/*!< Data type */
 	sccp_configurationchange_t change;					/*!< Does a change of this value needs a device restart */
 	const char *defaultValue;						/*!< Default value */
@@ -119,15 +127,15 @@ typedef struct SCCPConfigOption {
 #define sccp_config_parse_permithosts "(permithosts)=[[hostname]]"
 #define sccp_config_parse_addons "(addons)=7914,7915,7916,SPA500S"
 #define sccp_config_parse_privacyFeature "(privacyFeature)=full|on|off"
-#define sccp_config_parse_earlyrtp "(earlyrtp)=none|offhook|dial|ringout|progress"
-#define sccp_config_parse_dtmfmode "(dtmfmode)=outofband|inband"
-#define sccp_config_parse_mwilamp "(mwilamp)=on|off|wink|flash|blink"
+//#define sccp_config_parse_earlyrtp "(earlyrtp)=none|offhook|dial|ringout|progress"
+//#define sccp_config_parse_dtmfmode "(dtmfmode)=outofband|inband"
+//#define sccp_config_parse_mwilamp "(mwilamp)=on|off|wink|flash|blink"
 #define sccp_config_parse_debug "(debug)=all,none,core,sccp,hint,rtp,device,line,action,channel,cli,config,feature,feature_button,softkey,indicate,pbx,socket,mwi,event,adv_feature,conference,buttontemplate,speeddial,codec,realtime,lock,threadlock,message,newcode,high,myi,fixme,fyi"
 #define sccp_config_parse_ipaddress "(ipaddress)=[[ipaddress]]"
 #define sccp_config_parse_port "(port)=[[port]]"
-#define sccp_config_parse_blindtransferindication "(blindtransferindication)=moh|ring"
-#define sccp_config_parse_callanswerorder "(callanswerorder)=oldestfirst|lastfirst"
-#define sccp_config_parse_regcontext "(regcontext)=[[string]]"
+//#define sccp_config_parse_blindtransferindication "(blindtransferindication)=moh|ring"
+//#define sccp_config_parse_callanswerorder "(callanswerorder)=oldestfirst|lastfirst"
+//#define sccp_config_parse_regcontext "(regcontext)=[[string]]"
 #define sccp_config_parse_context "(context)=[[context]]"
 #define sccp_config_parse_hotline_context "(hotline_context)=[[context]]"
 #define sccp_config_parse_hotline_exten "(hotline_exten)=[[value]]"
@@ -136,9 +144,28 @@ typedef struct SCCPConfigOption {
 #define sccp_config_parse_jbflags_log "(jbflags_log)=on|off"
 #define sccp_config_parse_codec_preferences "(codec)=all|(alaw,ulaw,gsm,ilbc,g722,g723,g729,g726,slin,slin16)"
 #define sccp_config_parse_deny_permit "(permit)=[[ipaddress]],internal | (deny)=[[ipaddress]],internal"
-#define sccp_config_parse_dnd_wrapper "(dnd)=reject|silent|user|on|off"
+#define sccp_config_parse_hotline_label ""
+//#define sccp_config_parse_dnd_wrapper "(dnd)=reject|silent|user|on|off"
 
+uint32_t (sccp_enum_str2intval) (const char *lookup_str);
+const char *(sccp_enum_all_entries)(void);
 
+#define sccp_earlyrtp_str2intval sccp_enum_str2intval
+#define skinny_lampmode_str2intval sccp_enum_str2intval
+#define sccp_blindtransferindication_str2intval sccp_enum_str2intval
+#define sccp_call_answer_order_str2intval sccp_enum_str2intval
+#define sccp_dtmfmode_str2intval sccp_enum_str2intval
+#define sccp_dndmode_str2intval sccp_enum_str2intval
+#define sccp_nat_str2intval sccp_enum_str2intval
+
+#define sccp_earlyrtp_all_entries sccp_enum_all_entries
+#define skinny_lampmode_all_entries sccp_enum_all_entries
+#define sccp_blindtransferindication_all_entries sccp_enum_all_entries
+#define sccp_call_answer_order_all_entries sccp_enum_all_entries
+#define sccp_dtmfmode_all_entries sccp_enum_all_entries
+#define sccp_dndmode_all_entries sccp_enum_all_entries
+#define sccp_nat_all_entries sccp_enum_all_entries
+   
 #include "../../src/sccp_config_entries.hh"
 
 /*!
@@ -175,4 +202,3 @@ static const SCCPConfigSegment *sccp_find_segment(const sccp_config_segment_t se
 	}
 	return NULL;
 }
-
