@@ -7,20 +7,37 @@
  *              Modified by Jan Czmok and Julien Goodwin
  * \note        This program is free software and may be modified and distributed under the terms of the GNU Public License.
  *              See the LICENSE file at the top of the source tree.
- *
- * $Date: 2015-04-19 13:12:47 +0200 (Sun, 19 Apr 2015) $
- * $Revision: 6022 $  
  */
 #include <config.h>
-#include "../../common.h"
-#include "../../sccp_device.h"
-#include "../../sccp_channel.h"
-#include "../../sccp_utils.h"
-#include "../../sccp_indicate.h"
-#include "../../sccp_socket.h"
-#include "../../sccp_pbx.h"
+#include "common.h"
+#include "sccp_device.h"
+#include "sccp_channel.h"
+#include "sccp_utils.h"
+#include "sccp_indicate.h"
+#include "sccp_socket.h"
+#include "sccp_pbx.h"
+#include "sccp_line.h"
 
-SCCP_FILE_VERSION(__FILE__, "$Revision: 6022 $");
+SCCP_FILE_VERSION(__FILE__, "");
+
+#include <asterisk.h>
+#ifdef HAVE_PBX_ACL_H				// ast_str2cos ast_str2tos
+#  include <asterisk/acl.h>
+#endif
+#include <asterisk/causes.h>
+#ifdef HAVE_PBX_APP_H
+#  include <asterisk/app.h>
+#endif
+#include <asterisk/callerid.h>
+#include <asterisk/musiconhold.h>
+#include <asterisk/astdb.h>
+#ifdef HAVE_PBX_FEATURES_H
+#  include <asterisk/features.h>
+#endif
+#if ASTERISK_VERSION_GROUP >= 112
+#include <asterisk/features_config.h>
+#include <asterisk/pickup.h>
+#endif
 
 /*
  * \brief itterate through locked pbx channels
@@ -31,6 +48,7 @@ SCCP_FILE_VERSION(__FILE__, "$Revision: 6022 $");
  * \todo implement pbx_channel_walk_locked or replace
  * \deprecated
  */
+#if UNUSEDCODE // 2015-11-01
 PBX_CHANNEL_TYPE *pbx_channel_walk_locked(PBX_CHANNEL_TYPE * target)
 {
 #if ASTERISK_VERSION_NUMBER >= 10800
@@ -62,53 +80,7 @@ PBX_CHANNEL_TYPE *pbx_channel_walk_locked(PBX_CHANNEL_TYPE * target)
 	return ast_channel_walk_locked(target);
 #endif
 }
-
-/*
- * \brief iterate through locked pbx channels and search using specified match function
- * \param is_match match function
- * \param data paremeter data for match function
- * \return pbx_channel Locked Asterisk Channel
- *
- * \deprecated
- */
-PBX_CHANNEL_TYPE *pbx_channel_search_locked(int (*is_match) (PBX_CHANNEL_TYPE *, void *), void *data)
-{
-	//#ifdef ast_channel_search_locked
-#if ASTERISK_VERSION_NUMBER < 10800
-	return ast_channel_search_locked(is_match, data);
-#else
-	boolean_t matched = FALSE;
-	PBX_CHANNEL_TYPE *pbx_channel = NULL;
-	PBX_CHANNEL_TYPE *tmp = NULL;
-
-	struct ast_channel_iterator *iter = NULL;
-
-	if (!(iter = ast_channel_iterator_all_new())) {
-		return NULL;
-	}
-
-	for (; iter && (tmp = ast_channel_iterator_next(iter)); tmp = pbx_channel_unref(tmp)) {
-		pbx_channel_lock(tmp);
-		if (is_match(tmp, data)) {
-			matched = TRUE;
-			break;
-		}
-		pbx_channel_unlock(tmp);
-	}
-
-	if (iter) {
-		ast_channel_iterator_destroy(iter);
-	}
-
-	if (matched) {
-		pbx_channel = tmp;
-		tmp = pbx_channel_unref(tmp);
-		return pbx_channel;
-	} else {
-		return NULL;
-	}
 #endif
-}
 
 /************************************************************************************************************** CONFIG **/
 
@@ -123,7 +95,8 @@ PBX_CHANNEL_TYPE *pbx_channel_search_locked(int (*is_match) (PBX_CHANNEL_TYPE *,
  *
  * \deprecated
  */
-struct ast_ha *pbx_append_ha(NEWCONST char *sense, const char *stuff, struct ast_ha *path, int *error)
+#if UNUSEDCODE // 2015-11-01
+pbx_channel_walk_lockedstruct ast_ha *pbx_append_ha(NEWCONST char *sense, const char *stuff, struct ast_ha *path, int *error)
 {
 #if ASTERISK_VERSION_NUMBER < 10600
 	return ast_append_ha(sense, stuff, path);
@@ -131,6 +104,7 @@ struct ast_ha *pbx_append_ha(NEWCONST char *sense, const char *stuff, struct ast
 	return ast_append_ha(sense, stuff, path, error);
 #endif														// ASTERISK_VERSION_NUMBER
 }
+#endif
 
 /*
  * \brief Register a new context
@@ -231,51 +205,57 @@ static const struct dscp_codepoint dscp_pool1[] = {
 	{"EF", 0x2E},
 };
 
-int pbx_str2tos(const char *value, unsigned int *tos)
+int pbx_str2tos(const char *value, uint8_t *tos)
 {
 	int fval;
 
 	unsigned int x;
 
 	if (sscanf(value, "%30i", &fval) == 1) {
-		*tos = fval & 0xFF;
-		return 0;
+		*tos = (uint16_t)fval & 0xFF;
+		return *tos;
 	}
 
 	for (x = 0; x < ARRAY_LEN(dscp_pool1); x++) {
 		if (!strcasecmp(value, dscp_pool1[x].name)) {
-			*tos = dscp_pool1[x].space << 2;
-			return 0;
+			*tos = (uint8_t)dscp_pool1[x].space << 2;
+			return *tos;
 		}
 	}
 
-	return -1;
+	return 0;
 }
 #else
-int pbx_str2tos(const char *value, unsigned int *tos)
+int pbx_str2tos(const char *value, uint8_t *tos)
 {
-	return ast_str2tos(value, tos);
+	uint32_t tos_value = 0;
+	ast_str2tos(value, &tos_value);
+	*tos = (uint8_t) tos_value;
+	return *tos;
 }
 #endif														// ASTERISK_VERSION_NUMBER
 
 #if ASTERISK_VERSION_NUMBER < 10600
-int pbx_str2cos(const char *value, unsigned int *cos)
+int pbx_str2cos(const char *value, uint8_t *cos)
 {
 	int fval;
 
 	if (sscanf(value, "%30d", &fval) == 1) {
 		if (fval < 8) {
-			*cos = fval;
-			return 0;
+			*cos = (uint8_t)fval;
+			return *cos;
 		}
 	}
 
-	return -1;
+	return 0;
 }
 #else
-int pbx_str2cos(const char *value, unsigned int *cos)
+int pbx_str2cos(const char *value, uint8_t *cos)
 {
-	return ast_str2cos(value, cos);
+	uint32_t cos_value = 0;
+	ast_str2cos(value, &cos_value);
+	*cos = (uint8_t) cos_value;
+	return *cos;
 }
 #endif														// ASTERISK_VERSION_NUMBER
 
@@ -297,6 +277,7 @@ int pbx_str2cos(const char *value, unsigned int *cos)
  * 
  * @{
  */
+#if UNUSEDCODE // 2015-11-01
 int pbx_context_remove_extension(const char *context, const char *extension, int priority, const char *registrar)
 {
 #if ASTERISK_VERSION_NUMBER >= 10600
@@ -310,6 +291,7 @@ int pbx_context_remove_extension(const char *context, const char *extension, int
 	return ast_context_remove_extension(context, extension, priority, registrar);
 #endif														// ASTERISK_VERSION_NUMBER
 }
+#endif
 
 /*!   
  * \brief Send ack in manager list transaction
@@ -341,9 +323,9 @@ skinny_codec_t pbx_codec2skinny_codec(ast_format_type fmt)
 {
 	uint32_t i;
 
-	for (i = 1; i < ARRAY_LEN(skinny2pbx_codec_maps); i++) {
-		if (skinny2pbx_codec_maps[i].pbx_codec == (uint64_t) fmt) {
-			return skinny2pbx_codec_maps[i].skinny_codec;
+	for (i = 1; i < ARRAY_LEN(pbx2skinny_codec_maps); i++) {
+		if (pbx2skinny_codec_maps[i].pbx_codec == (uint64_t) fmt) {
+			return pbx2skinny_codec_maps[i].skinny_codec;
 		}
 	}
 	return 0;
@@ -360,9 +342,9 @@ ast_format_type skinny_codec2pbx_codec(skinny_codec_t codec)
 {
 	uint32_t i;
 
-	for (i = 1; i < ARRAY_LEN(skinny2pbx_codec_maps); i++) {
-		if (skinny2pbx_codec_maps[i].skinny_codec == codec) {
-			return skinny2pbx_codec_maps[i].pbx_codec;
+	for (i = 1; i < ARRAY_LEN(pbx2skinny_codec_maps); i++) {
+		if (pbx2skinny_codec_maps[i].skinny_codec == codec) {
+			return pbx2skinny_codec_maps[i].pbx_codec;
 		}
 	}
 	return 0;
@@ -375,13 +357,13 @@ ast_format_type skinny_codec2pbx_codec(skinny_codec_t codec)
  *
  * \return bit array fmt/Format of ast_format_type (int)
  */
-int skinny_codecs2pbx_codecs(skinny_codec_t * skinny_codecs)
+int skinny_codecs2pbx_codecs(skinny_codec_t * codecs)
 {
 	uint32_t i;
 	int res_codec = 0;
 
 	for (i = 1; i < SKINNY_MAX_CAPABILITIES; i++) {
-		res_codec |= skinny_codec2pbx_codec(skinny_codecs[i]);
+		res_codec |= skinny_codec2pbx_codec(codecs[i]);
 	}
 	return res_codec;
 }
@@ -444,9 +426,13 @@ static boolean_t sccp_wrapper_asterisk_carefullHangup(sccp_channel_t * c)
 	AUTO_RELEASE sccp_channel_t *channel = sccp_channel_retain(c);
 
 	if (channel && channel->owner) {
+		pbx_channel_lock(channel->owner);
 		PBX_CHANNEL_TYPE *pbx_channel = pbx_channel_ref(channel->owner);
+		pbx_channel_unlock(channel->owner);
 
-		sccp_channel_stop_and_deny_scheduled_tasks(channel);
+		if (ATOMIC_FETCH(&channel->scheduler.deny, &channel->scheduler.lock) == 0) {
+			sccp_channel_stop_and_deny_scheduled_tasks(channel);
+		}
 
 		/* let's wait for a bit, for the dust to settle */
 		sched_yield();
@@ -454,7 +440,15 @@ static boolean_t sccp_wrapper_asterisk_carefullHangup(sccp_channel_t * c)
 
 		/* recheck everything before going forward */
 		pbx_log(LOG_NOTICE, "%s: (sccp_wrapper_asterisk_carefullHangup) processing hangup request, using carefull version. Standby.\n", pbx_channel_name(pbx_channel));
-		if (!pbx_check_hangup(pbx_channel)) {
+		if (!pbx_channel || pbx_test_flag(pbx_channel_flags(pbx_channel), AST_FLAG_ZOMBIE) || pbx_check_hangup_locked(pbx_channel)) {
+			pbx_log(LOG_NOTICE, "%s: (sccp_wrapper_asterisk_carefullHangup) Already Hungup. Forcing SCCP Remove Call.\n", pbx_channel_name(pbx_channel));
+			AUTO_RELEASE sccp_device_t *d = sccp_channel_getDevice_retained(channel);
+
+			if (d) {
+				sccp_indicate(d, channel, SCCP_CHANNELSTATE_ONHOOK);
+			}
+			res = TRUE;
+		} else {
 			pbx_log(LOG_NOTICE, "%s: (sccp_wrapper_asterisk_carefullHangup) Channel still active.\n", pbx_channel_name(pbx_channel));
 			if (pbx_channel_pbx(pbx_channel) || pbx_test_flag(pbx_channel_flags(pbx_channel), AST_FLAG_BLOCKING) || pbx_channel_state(pbx_channel) == AST_STATE_UP) {
 				pbx_log(LOG_NOTICE, "%s: (sccp_wrapper_asterisk_carefullHangup) Has PBX -> ast_queue_hangup.\n", pbx_channel_name(pbx_channel));
@@ -464,14 +458,6 @@ static boolean_t sccp_wrapper_asterisk_carefullHangup(sccp_channel_t * c)
 				ast_hangup(pbx_channel);
 				res = TRUE;
 			}
-		} else {
-			pbx_log(LOG_NOTICE, "%s: (sccp_wrapper_asterisk_carefullHangup) Already Hungup. Forcing SCCP Remove Call.\n", pbx_channel_name(pbx_channel));
-			AUTO_RELEASE sccp_device_t *d = sccp_channel_getDevice_retained(channel);
-
-			if (d) {
-				sccp_indicate(d, channel, SCCP_CHANNELSTATE_ONHOOK);
-			}
-			res = TRUE;
 		}
 		pbx_channel_unref(pbx_channel);
 	}
@@ -484,21 +470,24 @@ boolean_t sccp_wrapper_asterisk_requestQueueHangup(sccp_channel_t * c)
 	AUTO_RELEASE sccp_channel_t *channel = sccp_channel_retain(c);
 
 	if (channel) {
-		PBX_CHANNEL_TYPE *pbx_channel = channel->owner;
+		PBX_CHANNEL_TYPE *pbx_channel = pbx_channel_ref(channel->owner);
 
-		sccp_channel_stop_and_deny_scheduled_tasks(channel);
+		if (ATOMIC_FETCH(&channel->scheduler.deny, &channel->scheduler.lock) == 0) {
+			sccp_channel_stop_and_deny_scheduled_tasks(channel);
+		}
 
 		channel->hangupRequest = sccp_wrapper_asterisk_carefullHangup;
-		if (!pbx_check_hangup(pbx_channel)) {								/* if channel is not already been hungup */
-			res = ast_queue_hangup(pbx_channel) ? FALSE : TRUE;
-		} else {
+		if (!pbx_channel || pbx_test_flag(pbx_channel_flags(pbx_channel), AST_FLAG_ZOMBIE) || pbx_check_hangup_locked(pbx_channel)) {
 			pbx_log(LOG_NOTICE, "%s: (sccp_wrapper_asterisk_requestQueueHangup) Already Hungup\n", channel->designator);
 			AUTO_RELEASE sccp_device_t *d = sccp_channel_getDevice_retained(channel);
 
 			if (d) {
 				sccp_indicate(d, channel, SCCP_CHANNELSTATE_ONHOOK);
 			}
+		} else {
+			res = ast_queue_hangup(pbx_channel) ? FALSE : TRUE;
 		}
+		pbx_channel_unref(pbx_channel);
 	}
 	return res;
 }
@@ -509,51 +498,52 @@ boolean_t sccp_wrapper_asterisk_requestHangup(sccp_channel_t * c)
 	AUTO_RELEASE sccp_channel_t *channel = sccp_channel_retain(c);
 
 	if (channel) {
-		PBX_CHANNEL_TYPE *pbx_channel = channel->owner;
-
-		sccp_channel_stop_and_deny_scheduled_tasks(channel);
+		PBX_CHANNEL_TYPE *pbx_channel = pbx_channel_ref(channel->owner);
 
+		if (ATOMIC_FETCH(&channel->scheduler.deny, &channel->scheduler.lock) == 0) {
+			sccp_channel_stop_and_deny_scheduled_tasks(channel);
+		}
 		channel->hangupRequest = sccp_wrapper_asterisk_carefullHangup;
-		if (!pbx_check_hangup(pbx_channel)) {
-			if (pbx_test_flag(pbx_channel_flags(pbx_channel), AST_FLAG_BLOCKING)) {
-				return sccp_wrapper_asterisk_requestQueueHangup(channel);
-			}
-			ast_hangup(pbx_channel);
-			res = TRUE;
-		} else {
+
+		if (!pbx_channel || pbx_test_flag(pbx_channel_flags(pbx_channel), AST_FLAG_ZOMBIE) || pbx_check_hangup_locked(pbx_channel)) {
 			AUTO_RELEASE sccp_device_t *d = sccp_channel_getDevice_retained(channel);
 
 			if (d) {
 				sccp_indicate(d, channel, SCCP_CHANNELSTATE_ONHOOK);
 			}
+		} else {
+			if (pbx_test_flag(pbx_channel_flags(pbx_channel), AST_FLAG_BLOCKING)) {
+				res = sccp_wrapper_asterisk_requestQueueHangup(channel);
+			} else {
+				ast_hangup(pbx_channel);
+				res = TRUE;
+			}
 		}
+		pbx_channel_unref(pbx_channel);
 	}
 	return res;
 }
 
 int sccp_asterisk_pbx_fktChannelWrite(PBX_CHANNEL_TYPE * ast, const char *funcname, char *args, const char *value)
 {
-	sccp_channel_t *c;
-	boolean_t res = TRUE;
+	int res = 0;
 
-	if (!(c = get_sccp_channel_from_pbx_channel(ast))) {
-		pbx_log(LOG_ERROR, "This function requires a valid SCCP channel\n");
-		return -1;
-	} else {
+	AUTO_RELEASE sccp_channel_t *c = get_sccp_channel_from_pbx_channel(ast);
+	if (c) {
 		if (!strcasecmp(args, "MaxCallBR")) {
 			sccp_log((DEBUGCAT_CORE)) (VERBOSE_PREFIX_3 "%s: set max call bitrate to %s\n", (char *) c->currentDeviceId, value);
 
 			if (sscanf(value, "%ud", &c->maxBitRate)) {
 				pbx_builtin_setvar_helper(ast, "_MaxCallBR", value);
-				res = TRUE;
 			} else {
-				res = FALSE;
+				res = -1;
 			}
 
 		} else if (!strcasecmp(args, "codec")) {
 			res = sccp_channel_setPreferredCodec(c, value);
 			
 		} else if (!strcasecmp(args, "video")) {
+			pbx_builtin_setvar_helper(ast, "_SCCP_VIDEO_MODE", value);
 			res = sccp_channel_setVideoMode(c, value);
 			
 		} else if (!strcasecmp(args, "CallingParty")) {
@@ -587,11 +577,13 @@ int sccp_asterisk_pbx_fktChannelWrite(PBX_CHANNEL_TYPE * ast, const char *funcna
 				c->setMicrophone(c, TRUE);
 			}
 		} else {
-			res = FALSE;
+			res = -1;
 		}
-		c = sccp_channel_release(c);
+	} else {
+		pbx_log(LOG_ERROR, "This function requires a valid SCCP channel\n");
+		res = -1;
 	}
-	return res ? 0 : -1;
+	return res;
 }
 
 /***** database *****/
@@ -654,7 +646,7 @@ boolean_t sccp_asterisk_removeTreeFromDatabase(const char *family, const char *k
  */
 int sccp_asterisk_moh_start(PBX_CHANNEL_TYPE * pbx_channel, const char *mclass, const char *interpclass)
 {
-	if (!ast_test_flag(pbx_channel_flags(pbx_channel), AST_FLAG_MOH)) {
+	if (!pbx_test_flag(pbx_channel_flags(pbx_channel), AST_FLAG_MOH)) {
 		pbx_set_flag(pbx_channel_flags(pbx_channel), AST_FLAG_MOH);
 		return ast_moh_start((PBX_CHANNEL_TYPE *) pbx_channel, mclass, interpclass);
 	} else {
@@ -664,7 +656,7 @@ int sccp_asterisk_moh_start(PBX_CHANNEL_TYPE * pbx_channel, const char *mclass,
 
 void sccp_asterisk_moh_stop(PBX_CHANNEL_TYPE * pbx_channel)
 {
-	if (ast_test_flag(pbx_channel_flags(pbx_channel), AST_FLAG_MOH)) {
+	if (pbx_test_flag(pbx_channel_flags(pbx_channel), AST_FLAG_MOH)) {
 		pbx_clear_flag(pbx_channel_flags(pbx_channel), AST_FLAG_MOH);
 		ast_moh_stop((PBX_CHANNEL_TYPE *) pbx_channel);
 	}
@@ -673,74 +665,170 @@ void sccp_asterisk_moh_stop(PBX_CHANNEL_TYPE * pbx_channel)
 void sccp_asterisk_redirectedUpdate(sccp_channel_t * channel, const void *data, size_t datalen)
 {
 	PBX_CHANNEL_TYPE *ast = channel->owner;
+	int redirectreason = 0;
+	sccp_callinfo_t *ci = sccp_channel_getCallInfo(channel);
 
+	sccp_callinfo_getter(ci, 
+		SCCP_CALLINFO_LAST_REDIRECT_REASON, &redirectreason,
+		SCCP_CALLINFO_KEY_SENTINEL);
 #if ASTERISK_VERSION_GROUP >106
 	struct ast_party_id redirecting_from = pbx_channel_redirecting_effective_from(ast);
 	struct ast_party_id redirecting_to = pbx_channel_redirecting_effective_to(ast);
 
-	sccp_log((DEBUGCAT_PBX)) (VERBOSE_PREFIX_3 "%s: Got redirecting update. From %s<%s>; To %s<%s>\n", pbx_channel_name(ast), (redirecting_from.name.valid && redirecting_from.name.str) ? redirecting_from.name.str : "", (redirecting_from.number.valid && redirecting_from.number.str) ? redirecting_from.number.str : "", (redirecting_to.name.valid && redirecting_to.name.str) ? redirecting_to.name.str : "",
-				  (redirecting_to.number.valid && redirecting_to.number.str) ? redirecting_to.number.str : "");
+	sccp_log((DEBUGCAT_PBX)) (VERBOSE_PREFIX_3 "%s: Got redirecting update. From %s<%s>; To %s<%s>\n", pbx_channel_name(ast),
+				(redirecting_from.name.valid && redirecting_from.name.str) ? redirecting_from.name.str : "", 
+				(redirecting_from.number.valid && redirecting_from.number.str) ? redirecting_from.number.str : "", 
+				(redirecting_to.name.valid && redirecting_to.name.str) ? redirecting_to.name.str : "",
+				(redirecting_to.number.valid && redirecting_to.number.str) ? redirecting_to.number.str : "");
 
-	if (redirecting_from.name.valid && redirecting_from.name.str) {
-		sccp_copy_string(channel->callInfo.lastRedirectingPartyName, redirecting_from.name.str, sizeof(channel->callInfo.callingPartyName));
-	}
+	sccp_callinfo_setter(ci, 
+		SCCP_CALLINFO_LAST_REDIRECTINGPARTY_NAME, redirecting_from.name.valid && redirecting_from.name.str ? redirecting_from.name.str : NULL, 
+		SCCP_CALLINFO_LAST_REDIRECTINGPARTY_NUMBER, (redirecting_from.number.valid && redirecting_from.number.str) ? redirecting_from.number.str : "",
+		SCCP_CALLINFO_ORIG_CALLEDPARTY_NUMBER, (redirecting_from.number.valid && redirecting_from.number.str) ? redirecting_from.number.str : "",
+		SCCP_CALLINFO_ORIG_CALLEDPARTY_NAME, redirecting_from.name.valid && redirecting_from.name.str ? redirecting_from.name.str : NULL,
+		SCCP_CALLINFO_ORIG_CALLEDPARTY_REDIRECT_REASON, redirectreason,
+		SCCP_CALLINFO_LAST_REDIRECT_REASON, 4,					// need to figure out these codes
+		SCCP_CALLINFO_KEY_SENTINEL);
 
-	sccp_copy_string(channel->callInfo.lastRedirectingPartyNumber, (redirecting_from.number.valid && redirecting_from.number.str) ? redirecting_from.number.str : "", sizeof(channel->callInfo.lastRedirectingPartyNumber));
-	channel->callInfo.lastRedirectingParty_valid = 1;
 #else
-	sccp_copy_string(channel->callInfo.lastRedirectingPartyNumber, ast->cid.cid_rdnis ? ast->cid.cid_rdnis : "", sizeof(channel->callInfo.lastRedirectingPartyNumber));
-	channel->callInfo.lastRedirectingParty_valid = 1;
+	sccp_log((DEBUGCAT_PBX)) (VERBOSE_PREFIX_3 "%s: Got redirecting update. From <%s>\n", pbx_channel_name(ast), ast->cid.cid_rdnis);
+	sccp_callinfo_setter(ci, 
+		SCCP_CALLINFO_LAST_REDIRECTINGPARTY_NUMBER, ast->cid.cid_rdnis ? ast->cid.cid_rdnis : "",
+		SCCP_CALLINFO_ORIG_CALLEDPARTY_NUMBER, ast->cid.cid_rdnis ? ast->cid.cid_rdnis : NULL,
+		SCCP_CALLINFO_ORIG_CALLEDPARTY_REDIRECT_REASON, redirectreason,
+		SCCP_CALLINFO_LAST_REDIRECT_REASON, 4,					// need to figure out these codes
+		SCCP_CALLINFO_KEY_SENTINEL);
 #endif
-	channel->callInfo.originalCdpnRedirectReason = channel->callInfo.lastRedirectingReason;
-	channel->callInfo.lastRedirectingReason = 4;								// need to figure out these codes
 
+	sccp_channel_display_callInfo(channel);
 	sccp_channel_send_callinfo2(channel);
 }
 
+/*!
+ * \brief Update Connected Line
+ * \param channel Asterisk Channel as ast_channel
+ * \param data Asterisk Data
+ * \param datalen Asterisk Data Length
+ */
+void sccp_asterisk_connectedline(sccp_channel_t * channel, const void *data, size_t datalen)
+{
+#if ASTERISK_VERSION_GROUP > 106
+	PBX_CHANNEL_TYPE *ast = channel->owner;
+	int changes = 0;
+	sccp_callinfo_t *const callInfo = sccp_channel_getCallInfo(channel);
+
+	sccp_log((DEBUGCAT_PBX)) (VERBOSE_PREFIX_3 "%s: Got connected line update, connected.id.number=%s, connected.id.name=%s, reason=%d\n", pbx_channel_name(ast), pbx_channel_connected_id(ast).number.str ? pbx_channel_connected_id(ast).number.str : "(nil)", pbx_channel_connected_id(ast).name.str ? pbx_channel_connected_id(ast).name.str : "(nil)", pbx_channel_connected_source(ast));
+
+	char tmpCallingNumber[StationMaxDirnumSize] = {0};
+	char tmpCallingName[StationMaxNameSize] = {0};
+	char tmpCalledNumber[StationMaxDirnumSize] = {0};
+	char tmpCalledName[StationMaxNameSize] = {0};
+	int tmpOrigCalledPartyRedirectReason = 0;
+	int tmpLastRedirectReason = 4;		/* \todo need to figure out more about these codes */
+
+	sccp_callinfo_getter(callInfo,
+		SCCP_CALLINFO_CALLINGPARTY_NUMBER, &tmpCallingNumber,
+		SCCP_CALLINFO_CALLINGPARTY_NAME, &tmpCallingName,
+		SCCP_CALLINFO_CALLEDPARTY_NUMBER, &tmpCalledNumber,
+		SCCP_CALLINFO_CALLEDPARTY_NAME, &tmpCalledName,
+		SCCP_CALLINFO_ORIG_CALLEDPARTY_REDIRECT_REASON, &tmpOrigCalledPartyRedirectReason,
+		SCCP_CALLINFO_KEY_SENTINEL);
+
+	/* set the original calling/called party if the reason is a transfer */
+	if (channel->calltype == SKINNY_CALLTYPE_INBOUND && (pbx_channel_connected_source(ast) == AST_CONNECTED_LINE_UPDATE_SOURCE_TRANSFER || pbx_channel_connected_source(ast) == AST_CONNECTED_LINE_UPDATE_SOURCE_TRANSFER_ALERTING)) {
+		sccp_log(DEBUGCAT_CHANNEL) ("SCCP: (connectedline) Destination\n");
+		changes = sccp_callinfo_setter(callInfo, 
+			SCCP_CALLINFO_CALLINGPARTY_NUMBER, pbx_channel_connected_id(ast).number.str,
+			SCCP_CALLINFO_CALLINGPARTY_NAME, pbx_channel_connected_id(ast).name.str,
+
+			SCCP_CALLINFO_ORIG_CALLINGPARTY_NUMBER, pbx_channel_connected_id(ast).number.str,
+			SCCP_CALLINFO_ORIG_CALLINGPARTY_NAME, pbx_channel_connected_id(ast).name.str,
+
+			SCCP_CALLINFO_ORIG_CALLEDPARTY_NUMBER, tmpCallingNumber,
+			SCCP_CALLINFO_ORIG_CALLEDPARTY_NAME, tmpCallingName,
+			SCCP_CALLINFO_ORIG_CALLEDPARTY_REDIRECT_REASON, tmpOrigCalledPartyRedirectReason,
+
+			SCCP_CALLINFO_LAST_REDIRECTINGPARTY_NUMBER, tmpCallingNumber,
+			SCCP_CALLINFO_LAST_REDIRECTINGPARTY_NAME, tmpCallingNumber,
+			SCCP_CALLINFO_LAST_REDIRECT_REASON, tmpLastRedirectReason,
+
+			SCCP_CALLINFO_KEY_SENTINEL);
+	} else {
+		if (channel->calltype == SKINNY_CALLTYPE_INBOUND) {
+#if ASTERISK_VERSION_GROUP >= 111
+			struct ast_party_id redirecting_orig = pbx_channel_redirecting_effective_orig(ast);
+			if (!redirecting_orig.name.valid && !redirecting_orig.number.valid) {
+				changes = sccp_callinfo_setter(callInfo,
+					SCCP_CALLINFO_CALLINGPARTY_NUMBER, pbx_channel_connected_id(ast).number.str,
+					SCCP_CALLINFO_CALLINGPARTY_NAME, pbx_channel_connected_id(ast).name.str,
+					SCCP_CALLINFO_KEY_SENTINEL);
+			} else {
+				changes = sccp_callinfo_setter(callInfo,
+					SCCP_CALLINFO_CALLINGPARTY_NUMBER, pbx_channel_connected_id(ast).number.str,
+					SCCP_CALLINFO_CALLINGPARTY_NAME, pbx_channel_connected_id(ast).name.str,
+					SCCP_CALLINFO_ORIG_CALLEDPARTY_NAME, redirecting_orig.name.valid ? ast_channel_redirecting(ast)->orig.name.str : "",
+					SCCP_CALLINFO_ORIG_CALLEDPARTY_NUMBER, redirecting_orig.number.valid ? ast_channel_redirecting(ast)->orig.number.str : "",
+					SCCP_CALLINFO_KEY_SENTINEL);
+			}
+#else
+			changes = sccp_callinfo_setter(callInfo,
+				SCCP_CALLINFO_CALLINGPARTY_NUMBER, pbx_channel_connected_id(ast).number.str,
+				SCCP_CALLINFO_CALLINGPARTY_NAME, pbx_channel_connected_id(ast).name.str,
+				SCCP_CALLINFO_KEY_SENTINEL);
+#endif
+		} else {
+			changes = sccp_callinfo_setter(callInfo,
+				SCCP_CALLINFO_CALLEDPARTY_NUMBER, pbx_channel_connected_id(ast).number.str,
+				SCCP_CALLINFO_CALLEDPARTY_NAME, pbx_channel_connected_id(ast).name.str,
+				SCCP_CALLINFO_KEY_SENTINEL);
+		}
+
+
+	}
+	sccp_channel_display_callInfo(channel);
+	sccp_channel_send_callinfo2(channel);
+
+	if (changes) {								/* only send indications if something changed */
+		AUTO_RELEASE sccp_device_t *d = sccp_channel_getDevice_retained(channel);
+		sccp_indicate(d, channel, channel->state);
+	}
+#endif
+}
+
 void sccp_asterisk_sendRedirectedUpdate(const sccp_channel_t * channel, const char *fromNumber, const char *fromName, const char *toNumber, const char *toName, uint8_t reason)
 {
 #if ASTERISK_VERSION_GROUP >106
 	struct ast_party_redirecting redirecting;
 	struct ast_set_party_redirecting update_redirecting;
 
+	sccp_log((DEBUGCAT_PBX)) (VERBOSE_PREFIX_3 "%s: Send Redirected Update. From %s<%s>, To: %s<%s>\n", channel->designator, fromName, fromNumber, toName, toNumber);
+
 	ast_party_redirecting_init(&redirecting);
 	memset(&update_redirecting, 0, sizeof(update_redirecting));
 
 	/* update redirecting info line for source part */
-	if (!sccp_strlen_zero(redirecting.from.number.str)) {
-		ast_free(redirecting.from.number.str );
-	}
 	if (fromNumber) {
 		update_redirecting.from.number = 1;
 		redirecting.from.number.valid = 1;
-		redirecting.from.number.str = strdup(fromNumber);
+		redirecting.from.number.str = pbx_strdup(fromNumber);
 	}
 
-	if (!sccp_strlen_zero(redirecting.from.name.str)) {
-		ast_free(redirecting.from.name.str );
-	}
 	if (fromName) {
 		update_redirecting.from.name = 1;
 		redirecting.from.name.valid = 1;
-		redirecting.from.name.str = strdup(fromName);
+		redirecting.from.name.str = pbx_strdup(fromName);
 	}
 
-	if (!sccp_strlen_zero(redirecting.to.number.str)) {
-		ast_free(redirecting.to.number.str );
-	}
 	if (toNumber) {
 		update_redirecting.to.number = 1;
 		redirecting.to.number.valid = 1;
-		redirecting.to.number.str = strdup(toNumber);
+		redirecting.to.number.str = pbx_strdup(toNumber);
 	}
 
-	if (!sccp_strlen_zero(redirecting.to.name.str)) {
-		ast_free(redirecting.to.name.str );
-	}
 	if (toName) {
 		update_redirecting.to.name = 1;
 		redirecting.to.name.valid = 1;
-		redirecting.to.name.str = strdup(toName);
+		redirecting.to.name.str = pbx_strdup(toName);
 	}
 #if ASTERISK_VERSION_GROUP >111
 	redirecting.reason.code = reason;
@@ -756,18 +844,78 @@ void sccp_asterisk_sendRedirectedUpdate(const sccp_channel_t * channel, const ch
 		if (channel->owner->cid.cid_rdnis) {
 			ast_free(channel->owner->cid.cid_rdnis);
 		}
-		channel->owner->cid.cid_rdnis = ast_strdup(fromNumber);
+		channel->owner->cid.cid_rdnis = pbx_strdup(fromNumber);
 	}
 	// where is the call going to now
 	if (toNumber) {
 		if (channel->owner->cid.cid_dnid) {
 			ast_free(channel->owner->cid.cid_dnid);
 		}
-		channel->owner->cid.cid_dnid = ast_strdup(toNumber);
+		channel->owner->cid.cid_dnid = pbx_strdup(toNumber);
 	}
 #endif
 }
 
+/*!
+ * parse the DIAL options and store results by ref
+ */
+int sccp_parse_dial_options(char *options, sccp_autoanswer_t *autoanswer_type, uint8_t *autoanswer_cause, skinny_ringtype_t *ringermode)
+{
+	int res = 0;
+	int optc = 0;
+	char *optv[2];
+	int opti = 0;
+
+	/* parse options */
+	if (options && (optc = sccp_app_separate_args(options, '/', optv, sizeof(optv) / sizeof(optv[0])))) {
+		for (opti = 0; opti < optc; opti++) {
+			if (!strncasecmp(optv[opti], "aa", 2)) {
+				/* let's use the old style auto answer aa1w and aa2w */
+				if (!strncasecmp(optv[opti], "aa1w", 4)) {
+					*autoanswer_type = SCCP_AUTOANSWER_1W;
+					optv[opti] += 4;
+				} else if (!strncasecmp(optv[opti], "aa2w", 4)) {
+					*autoanswer_type = SCCP_AUTOANSWER_2W;
+					optv[opti] += 4;
+				} else if (!strncasecmp(optv[opti], "aa=", 3)) {
+					optv[opti] += 3;
+					pbx_log(LOG_NOTICE, "parsing aa\n");
+					if (!strncasecmp(optv[opti], "1w", 2)) {
+						*autoanswer_type = SCCP_AUTOANSWER_1W;
+						optv[opti] += 2;
+					} else if (!strncasecmp(optv[opti], "2w", 2)) {
+						*autoanswer_type = SCCP_AUTOANSWER_2W;
+						pbx_log(LOG_NOTICE, "set aa to 2w\n");
+						optv[opti] += 2;
+					}
+				}
+
+				/* since the pbx ignores autoanswer_cause unless SCCP_RWLIST_GETSIZE(&l->channels) > 1, it is safe to set it if provided */
+				if (!sccp_strlen_zero(optv[opti]) && (autoanswer_cause)) {
+					if (!strcasecmp(optv[opti], "b")) {
+						*autoanswer_cause = AST_CAUSE_BUSY;
+					} else if (!strcasecmp(optv[opti], "u")) {
+						*autoanswer_cause = AST_CAUSE_REQUESTED_CHAN_UNAVAIL;
+					} else if (!strcasecmp(optv[opti], "c")) {
+						*autoanswer_cause = AST_CAUSE_CONGESTION;
+					}
+				}
+				/* check for ringer options */
+			} else if (!strncasecmp(optv[opti], "ringer=", 7)) {
+				optv[opti] += 7;
+				*ringermode = skinny_ringtype_str2val(optv[opti]);
+			} else {
+				pbx_log(LOG_WARNING, "SCCP: Unknown option %s\n", optv[opti]);
+				res = -1;
+			}
+		}
+	}
+	if (*ringermode == SKINNY_RINGTYPE_SENTINEL) {
+		*ringermode = SKINNY_RINGTYPE_OUTSIDE;
+	}
+	return res;
+}
+
 /*!
  * \brief ACF Channel Read callback
  *
@@ -782,11 +930,9 @@ void sccp_asterisk_sendRedirectedUpdate(const sccp_channel_t * channel, const ch
  */
 int sccp_wrapper_asterisk_channel_read(PBX_CHANNEL_TYPE * ast, NEWCONST char *funcname, char *preparse, char *buf, size_t buflen)
 {
-	sccp_channel_t *c = NULL;
-	sccp_device_t *d = NULL;
 	int res = 0;
 
-	char *parse = sccp_strdupa(preparse);
+	char *parse = pbx_strdupa(preparse);
 
 	AST_DECLARE_APP_ARGS(args, AST_APP_ARG(param); AST_APP_ARG(type); AST_APP_ARG(field););
 	AST_STANDARD_APP_ARGS(args, parse);
@@ -796,12 +942,24 @@ int sccp_wrapper_asterisk_channel_read(PBX_CHANNEL_TYPE * ast, NEWCONST char *fu
 		return -1;
 	}
 
-	if ((c = get_sccp_channel_from_pbx_channel(ast))) {
-		if ((d = sccp_channel_getDevice_retained(c))) {
+	AUTO_RELEASE sccp_channel_t *c = get_sccp_channel_from_pbx_channel(ast);
+	if (c) {
+		AUTO_RELEASE sccp_device_t *d = sccp_channel_getDevice_retained(c);
+		if (d) {
 			if (!strcasecmp(args.param, "peerip")) {
-				sccp_copy_string(buf, sccp_socket_stringify(&d->session->sin), buflen);
+				struct sockaddr_storage sas = { 0 };
+				if (sccp_session_getOurIP(d->session, &sas, 0)) {
+					sccp_copy_string(buf, sccp_socket_stringify(&sas), buflen);
+				} else {
+					sccp_copy_string(buf, "--", buflen);
+				}
 			} else if (!strcasecmp(args.param, "recvip")) {
-				ast_copy_string(buf, sccp_socket_stringify(&d->session->sin), buflen);
+				struct sockaddr_storage sas = { 0 };
+				if (sccp_session_getSas(d->session, &sas)) {
+					sccp_copy_string(buf, sccp_socket_stringify(&sas), buflen);
+				} else {
+					sccp_copy_string(buf, "--", buflen);
+				}
 			} else if (!strcasecmp(args.param, "useragent")) {
 				sccp_copy_string(buf, skinny_devicetype2str(d->skinny_type), buflen);
 			} else if (!strcasecmp(args.param, "from")) {
@@ -815,12 +973,12 @@ int sccp_wrapper_asterisk_channel_read(PBX_CHANNEL_TYPE * ast, NEWCONST char *fu
 				}
 
 				if (sccp_strcaseequals(args.type, "audio")) {
-					rtp = c->rtp.audio.rtp;
+					rtp = c->rtp.audio.instance;
 				} else if (sccp_strcaseequals(args.type, "video")) {
-					rtp = c->rtp.video.rtp;
+					rtp = c->rtp.video.instance;
 				/*
 				} else if (sccp_strcaseequals(args.type, "text")) {
-					rtp = c->rtp.text.rtp;
+					rtp = c->rtp.text.instance;
 				*/
 				} else {
 					return -1;
@@ -839,74 +997,48 @@ int sccp_wrapper_asterisk_channel_read(PBX_CHANNEL_TYPE * ast, NEWCONST char *fu
 					} else {
 						struct ast_rtp_instance_stats stats;
 						int i;
+						enum __int_double { __INT, __DBL };
 						struct {
 							const char *name;
-							enum { INT, DBL } type;
+							enum __int_double type;
 							union {
 								unsigned int *i4;
 								double *d8;
 							};
 						} lookup[] = {
-							{
-								"txcount", INT, {
-							.i4 = &stats.txcount,},}, {
-								"rxcount", INT, {
-							.i4 = &stats.rxcount,},}, {
-								"txjitter", DBL, {
-							.d8 = &stats.txjitter,},}, {
-								"rxjitter", DBL, {
-							.d8 = &stats.rxjitter,},}, {
-								"remote_maxjitter", DBL, {
-							.d8 = &stats.remote_maxjitter,},}, {
-								"remote_minjitter", DBL, {
-							.d8 = &stats.remote_minjitter,},}, {
-								"remote_normdevjitter", DBL, {
-							.d8 = &stats.remote_normdevjitter,},}, {
-								"remote_stdevjitter", DBL, {
-							.d8 = &stats.remote_stdevjitter,},}, {
-								"local_maxjitter", DBL, {
-							.d8 = &stats.local_maxjitter,},}, {
-								"local_minjitter", DBL, {
-							.d8 = &stats.local_minjitter,},}, {
-								"local_normdevjitter", DBL, {
-							.d8 = &stats.local_normdevjitter,},}, {
-								"local_stdevjitter", DBL, {
-							.d8 = &stats.local_stdevjitter,},}, {
-								"txploss", INT, {
-							.i4 = &stats.txploss,},}, {
-								"rxploss", INT, {
-							.i4 = &stats.rxploss,},}, {
-								"remote_maxrxploss", DBL, {
-							.d8 = &stats.remote_maxrxploss,},}, {
-								"remote_minrxploss", DBL, {
-							.d8 = &stats.remote_minrxploss,},}, {
-								"remote_normdevrxploss", DBL, {
-							.d8 = &stats.remote_normdevrxploss,},}, {
-								"remote_stdevrxploss", DBL, {
-							.d8 = &stats.remote_stdevrxploss,},}, {
-								"local_maxrxploss", DBL, {
-							.d8 = &stats.local_maxrxploss,},}, {
-								"local_minrxploss", DBL, {
-							.d8 = &stats.local_minrxploss,},}, {
-								"local_normdevrxploss", DBL, {
-							.d8 = &stats.local_normdevrxploss,},}, {
-								"local_stdevrxploss", DBL, {
-							.d8 = &stats.local_stdevrxploss,},}, {
-								"rtt", DBL, {
-							.d8 = &stats.rtt,},}, {
-								"maxrtt", DBL, {
-							.d8 = &stats.maxrtt,},}, {
-								"minrtt", DBL, {
-							.d8 = &stats.minrtt,},}, {
-								"normdevrtt", DBL, {
-							.d8 = &stats.normdevrtt,},}, {
-								"stdevrtt", DBL, {
-							.d8 = &stats.stdevrtt,},}, {
-								"local_ssrc", INT, {
-							.i4 = &stats.local_ssrc,},}, {
-								"remote_ssrc", INT, {
-							.i4 = &stats.remote_ssrc,},}, {
-						NULL,},};
+							/* *INDENT-OFF* */
+							{"txcount", 		__INT, {.i4 = &stats.txcount},}, 
+							{"rxcount", 		__INT, {.i4 = &stats.rxcount,},}, 
+							{"txjitter", 		__DBL, {.d8 = &stats.txjitter,},}, 
+							{"rxjitter", 		__DBL, {.d8 = &stats.rxjitter,},},
+							{"remote_maxjitter", 	__DBL, {.d8 = &stats.remote_maxjitter,},},
+							{"remote_minjitter", 	__DBL, {.d8 = &stats.remote_minjitter,},},
+							{"remote_normdevjitter",__DBL, {.d8 = &stats.remote_normdevjitter,},},
+							{"remote_stdevjitter", 	__DBL, {.d8 = &stats.remote_stdevjitter,},},
+							{"local_maxjitter",	__DBL, {.d8 = &stats.local_maxjitter,},},
+							{"local_minjitter", 	__DBL, {.d8 = &stats.local_minjitter,},},
+							{"local_normdevjitter", __DBL, {.d8 = &stats.local_normdevjitter,},},
+							{"local_stdevjitter", 	__DBL, {.d8 = &stats.local_stdevjitter,},},
+							{"txploss", 		__INT, {.i4 = &stats.txploss,},},
+							{"rxploss", 		__INT, {.i4 = &stats.rxploss,},},
+							{"remote_maxrxploss", 	__DBL, {.d8 = &stats.remote_maxrxploss,},},
+							{"remote_minrxploss", 	__DBL, {.d8 = &stats.remote_minrxploss,},},
+							{"remote_normdevrxploss",__DBL, {.d8 = &stats.remote_normdevrxploss,},},
+							{"remote_stdevrxploss", __DBL, {.d8 = &stats.remote_stdevrxploss,},},
+							{"local_maxrxploss", 	__DBL, {.d8 = &stats.local_maxrxploss,},},
+							{"local_minrxploss", 	__DBL, {.d8 = &stats.local_minrxploss,},},
+							{"local_normdevrxploss",__DBL, {.d8 = &stats.local_normdevrxploss,},},
+							{"local_stdevrxploss", 	__DBL, {.d8 = &stats.local_stdevrxploss,},},
+							{"rtt", 		__DBL, {.d8 = &stats.rtt,},},
+							{"maxrtt", 		__DBL, {.d8 = &stats.maxrtt,},},
+							{"minrtt", 		__DBL, {.d8 = &stats.minrtt,},},
+							{"normdevrtt", 		__DBL, {.d8 = &stats.normdevrtt,},},
+							{"stdevrtt", 		__DBL, {.d8 = &stats.stdevrtt,},},
+							{"local_ssrc", 		__INT, {.i4 = &stats.local_ssrc,},},
+							{"remote_ssrc", 	__INT, {.i4 = &stats.remote_ssrc,},},
+							{NULL,},
+							/* *INDENT-ON* */
+						};
 
 						if (ast_rtp_instance_get_stats(rtp, &stats, AST_RTP_INSTANCE_STAT_ALL)) {
 							return -1;
@@ -914,7 +1046,7 @@ int sccp_wrapper_asterisk_channel_read(PBX_CHANNEL_TYPE * ast, NEWCONST char *fu
 
 						for (i = 0; !sccp_strlen_zero(lookup[i].name); i++) {
 							if (sccp_strcaseequals(args.field, lookup[i].name)) {
-								if (lookup[i].type == INT) {
+								if (lookup[i].type == __INT) {
 									snprintf(buf, buflen, "%u", *lookup[i].i4);
 								} else {
 									snprintf(buf, buflen, "%f", *lookup[i].d8);
@@ -931,65 +1063,56 @@ int sccp_wrapper_asterisk_channel_read(PBX_CHANNEL_TYPE * ast, NEWCONST char *fu
 			} else {
 				res = -1;
 			}
-			d = sccp_device_release(d);
 		} else {
 			res = -1;
 		}
-		c = sccp_channel_release(c);
 	} else {
 		res = -1;
 	}
 	return res;
 }
 
+/*!
+ * \brief Call asterisk automon feature
+ * \obsolete, using sccp_manager_action2str method instead
+ */
 boolean_t sccp_wrapper_asterisk_featureMonitor(const sccp_channel_t * channel)
 {
 #if ASTERISK_VERSION_GROUP >= 112
-	char *featexten;
+	char featexten[SCCP_MAX_EXTENSION] = "";
 
-	if (PBX(getFeatureExtension) (channel, &featexten)) {
+	if (iPbx.getFeatureExtension(channel, "automon", featexten) && !sccp_strlen_zero(featexten)) {
+		pbx_log(LOG_ERROR, "%s: Sending DTMF:'%s' to switch Monitor Feature\n", channel->designator, featexten);
 		struct ast_frame f = { AST_FRAME_DTMF, };
-		int j;
+		uint j;
 
 		f.len = 100;
 		for (j = 0; j < strlen(featexten); j++) {
 			f.subclass.integer = featexten[j];
 			ast_queue_frame(channel->owner, &f);
 		}
-		sccp_free(featexten);
 		return TRUE;
 	}
+	pbx_log(LOG_ERROR, "SCCP: Monitor Feature Extension Not available\n");
+	return FALSE;
 #else
-#ifdef CS_EXPERIMENTAL		// Added 2015/01/24
 	ast_rdlock_call_features();
-	
-	struct ast_call_feature *feature = ast_find_call_feature("automixmon");
+	struct ast_call_feature *feature = ast_find_call_feature("automon");
+	ast_unlock_call_features();
 
 	if (feature) {
-		struct ast_call_feature feat;
-		memcpy(&feat, feature, sizeof(feat));
-		ast_unlock_call_features();
-		PBX_CHANNEL_TYPE *bridgePeer = PBX(get_bridged_channel)(channel->owner);
+		PBX_CHANNEL_TYPE *bridgePeer = iPbx.get_bridged_channel(channel->owner);
 		if (bridgePeer) {
-			feat.operation(channel->owner, bridgePeer, NULL, "monitor button", FEATURE_SENSE_CHAN | FEATURE_SENSE_PEER, NULL);
+			feature->operation(channel->owner, bridgePeer, NULL, "monitor button", FEATURE_SENSE_CHAN | FEATURE_SENSE_PEER, NULL);
 			pbx_channel_unref(bridgePeer);
 		} else {
 			pbx_log(LOG_ERROR, "SCCP: No bridgepeer available\n");
 		}
 		return TRUE;
 	}
-#else				// Old Impl
-	struct ast_call_feature *feature = ast_find_call_feature("automon");
-
-	if (feature) {
-		feature->operation(channel->owner, channel->owner, NULL, "monitor button", 0, NULL);
-		return TRUE;
-	}
-#endif
-#endif
 	sccp_log(DEBUGCAT_CORE) (VERBOSE_PREFIX_3 "%s: Automon not available in features.conf/n", channel->designator);
 	return FALSE;
-
+#endif
 }
 
 #if !defined(AST_DEFAULT_EMULATE_DTMF_DURATION)
@@ -1069,32 +1192,33 @@ static int sccp_asterisk_doPickup(PBX_CHANNEL_TYPE * pbx_channel)
 enum ast_pbx_result pbx_pbx_start(PBX_CHANNEL_TYPE * pbx_channel)
 {
 	enum ast_pbx_result res = AST_PBX_FAILED;
-	sccp_channel_t *channel = NULL;
 
 	if (!pbx_channel) {
 		pbx_log(LOG_ERROR, "SCCP: (pbx_pbx_start) called without pbx channel\n");
 		return res;
 	}
 
-	if ((channel = get_sccp_channel_from_pbx_channel(pbx_channel))) {
+	AUTO_RELEASE sccp_channel_t *channel = get_sccp_channel_from_pbx_channel(pbx_channel);
+	if (channel) {
 		ast_channel_lock(pbx_channel);
 #if ASTERISK_VERSION_GROUP >= 111
+#if !defined(CS_AST_CHANNEL_CALLID_TYPEDEF)
 		struct ast_callid *callid = NULL;
-
+#else
+		ast_callid callid = 0;
+#endif
 		channel->pbx_callid_created = ast_callid_threadstorage_auto(&callid);
 		ast_channel_callid_set(pbx_channel, callid);
 #endif
 		// check if the pickup extension was entered
-		const char *dialedNumber = PBX(getChannelExten) (channel);
-		char *pickupexten;
+		const char *dialedNumber = iPbx.getChannelExten(channel);
+		char pickupexten[SCCP_MAX_EXTENSION];
 
-		if (PBX(getPickupExtension) (channel, &pickupexten) && sccp_strequals(dialedNumber, pickupexten)) {
+		if (iPbx.getPickupExtension(channel, pickupexten) && sccp_strequals(dialedNumber, pickupexten)) {
 			if (sccp_asterisk_doPickup(pbx_channel)) {
 				res = AST_PBX_SUCCESS;
 			}
 			ast_channel_unlock(pbx_channel);
-			channel = sccp_channel_release(channel);
-			sccp_free(pickupexten);
 			goto EXIT;
 		}
 		// channel->hangupRequest = sccp_wrapper_asterisk_dummyHangup;
@@ -1114,7 +1238,6 @@ enum ast_pbx_result pbx_pbx_start(PBX_CHANNEL_TYPE * pbx_channel)
 			}
 		}
 		ast_channel_unlock(pbx_channel);
-		channel = sccp_channel_release(channel);
 	}
 EXIT:
 	return res;
