@@ -4,12 +4,9 @@
  * \author      Diederik de Groot <ddegroot [at] users.sourceforge.net>
  * \note        This program is free software and may be modified and distributed under the terms of the GNU Public License.
  *              See the LICENSE file at the top of the source tree.
- *
- * $Date: 2015-04-03 03:51:36 +0200 (Fri, 03 Apr 2015) $
- * $Revision: 5995 $  
  */
-#ifndef SCCP_AST_DEFINE_H_
-#define SCCP_AST_DEFINE_H_
+#pragma once
+#include <config.h>
 
 // type redefinitions
 #define pbx_variable ast_variable
@@ -19,7 +16,7 @@
 #define sccp_malloc ast_malloc
 #define sccp_calloc ast_calloc
 #define sccp_realloc ast_realloc
-#define sccp_strdup ast_strdup
+//#define sccp_strdup ast_strdup
 #define sccp_free(x) {ast_free((void *)x); (x) = NULL; }
 #define sccp_asprintf ast_asprintf
 #define sccp_vasprintf ast_vasprintf
@@ -27,6 +24,31 @@
 #define sccp_alloca(size) __builtin_alloca(size)
 #define CS_BRIDGEPEERNAME "BRIDGEPEER"
 
+#ifdef SCANBUILD
+#define pbx_assert assert
+#else
+#define pbx_assert ast_assert
+#endif
+
+/* Lock Macro's */
+#define sccp_mutex_init(x)          		pbx_mutex_init(x)
+#define sccp_mutex_destroy(x)       		pbx_mutex_destroy(x)
+#define sccp_mutex_lock(x)			pbx_mutex_lock(x)
+#define sccp_mutex_lock_desc(x,y) 		pbx_mutex_lock(x)
+#define sccp_mutex_unlock(x)			pbx_mutex_unlock(x)
+#define sccp_mutex_trylock(x)			pbx_mutex_trylock(x)
+
+// SCCP_FILE_VERSION definition
+#if defined(LOW_MEMORY)
+#define SCCP_FILE_VERSION(_file, _version)
+#else
+#if CS_AST_REGISTER_FILE_VERSION
+#define SCCP_FILE_VERSION(_file, _version) ASTERISK_FILE_VERSION(_file, _version)
+#else
+#define SCCP_FILE_VERSION(_file, _version) ASTERISK_REGISTER_FILE()
+#endif
+#endif
+
 // codec / format redefinitions
 #define pbx_codec_pref_index ast_codec_pref_index
 #define pbx_codec_pref_getsize ast_codec_pref_getsize
@@ -43,11 +65,13 @@
 
 // general redefinitions
 #define pbx_check_hangup ast_check_hangup
+#define pbx_check_hangup_locked ast_check_hangup_locked
 #define pbx_channel_lock ast_channel_lock
 #define pbx_channel_unlock ast_channel_unlock
 #define pbx_sched_wait ast_sched_wait
 #define pbx_io_wait ast_io_wait
 #define pbx_gethostbyname ast_gethostbyname
+#define pbx_get_version ast_get_version
 #define pbx_apply_ha ast_apply_ha
 #define pbx_moh_stop ast_moh_stop
 #define pbx_channel_alloc ast_channel_alloc
@@ -64,12 +88,12 @@
 #define pbx_bridge_features_init ast_bridge_features_init
 
 #if ASTERISK_VERSION_NUMBER < 11010
-#define pbx_bridge_join(_bridge, _channel, _replace, _features, _tech_optimizations) ast_bridge_join(_bridge, _channel, _replace, _features)
+#define pbx_bridge_join(_bridge, _channel, _replace, _features, _tech_optimizations, _flags) ast_bridge_join(_bridge, _channel, _replace, _features)
 #else
 #ifndef CS_BRIDGE_JOIN_PASSREFERENCE
-#define pbx_bridge_join(_bridge, _channel, _replace, _features, _tech_optimizations) ast_bridge_join(_bridge, _channel, _replace, _features, _tech_optimizations)
+#define pbx_bridge_join(_bridge, _channel, _replace, _features, _tech_optimizations, _flags) ast_bridge_join(_bridge, _channel, _replace, _features, _tech_optimizations)
 #else
-#define pbx_bridge_join(_bridge, _channel, _replace, _features, _tech_optimizations) ast_bridge_join(_bridge, _channel, _replace, _features, _tech_optimizations, 0)
+#define pbx_bridge_join(_bridge, _channel, _replace, _features, _tech_optimizations, _flags) ast_bridge_join(_bridge, _channel, _replace, _features, _tech_optimizations, _flags)
 #endif
 #endif
 
@@ -79,6 +103,7 @@
 #define pbx_bridge_new(_a,_b,_c,_d,_e) ast_bridge_base_new(_a, _b, _c, _d, _e)
 #endif
 #define pbx_bridge_remove ast_bridge_remove
+#define pbx_bridge_kick ast_bridge_kick
 #define pbx_bridge_result ast_bridge_result
 
 #if ASTERISK_VERSION_GROUP < 110
@@ -95,6 +120,10 @@
 #define pbx_bridge_destroy(_x, _y) ast_bridge_destroy(_x)
 #define pbx_bridge_features_cleanup ast_bridge_features_cleanup
 #define pbx_bridge_change_state ast_bridge_change_state
+#define pbx_bridge_lock ast_bridge_lock
+#define pbx_bridge_unlock ast_bridge_unlock
+#define pbx_bridge_suspend ast_bridge_suspend
+#define pbx_bridge_unsuspend ast_bridge_unsuspend
 
 #define pbx_request ast_request
 #define pbx_build_string ast_build_string
@@ -240,7 +269,6 @@ typedef struct ast_event pbx_event_t;
 #define pbx_queue_hangup ast_queue_hangup
 #define pbx_random ast_random
 #define pbx_realloc ast_realloc
-#define pbx_register_file_version ast_register_file_version
 #define pbx_rtp_bridge ast_rtp_bridge
 #define pbx_rtp_codecs_payloads_set_rtpmap_type_rate ast_rtp_codecs_payloads_set_rtpmap_type_rate
 #define pbx_rtp_glue_register ast_rtp_glue_register
@@ -279,6 +307,7 @@ typedef struct ast_event pbx_event_t;
 #define pbx_sockaddr_port ast_sockaddr_port
 #define pbx_sockaddr_to_sin ast_sockaddr_to_sin
 #define pbx_state2str ast_state2str
+#define pbx_str_t struct ast_str
 #define pbx_str_alloca ast_str_alloca
 #define pbx_str_append ast_str_append
 #define pbx_str_append_escapecommas ast_str_append_escapecommas
@@ -288,6 +317,10 @@ typedef struct ast_event pbx_event_t;
 #define pbx_stream_and_wait ast_stream_and_wait
 #define pbx_say_number ast_say_number
 #define pbx_fileexists ast_fileexists
+#if ASTERISK_VERSION_GROUP < 111 && defined(__clang__)
+#undef AST_STRING_FIELD_ALLOCATION
+#define AST_STRING_FIELD_ALLOCATION(x) *((ast_string_field_allocation *) ((ast_string_field_allocation)x - __alignof__(ast_string_field_allocation)))
+#endif
 #define pbx_string_field_build ast_string_field_build
 #define pbx_string_field_set ast_string_field_set
 #define pbx_strip ast_strip
@@ -295,7 +328,10 @@ typedef struct ast_event pbx_event_t;
 #define pbx_true ast_true
 #define pbx_false ast_false
 #define pbx_tvnow ast_tvnow
+#if CS_AST_REGISTER_FILE_VERSION
+#define pbx_register_file_version ast_register_file_version
 #define pbx_unregister_file_version ast_unregister_file_version
+#endif
 #define pbx_update_use_count ast_update_use_count
 #define pbx_variable_browse ast_variable_browse
 #define pbx_variable_new ast_variable_new
@@ -350,6 +386,14 @@ typedef struct ast_event pbx_event_t;
 #define pbx_bridge_change_state ast_bridge_change_state
 #define pbx_channel_redirecting_effective_from(_a) (_a)->redirecting.from
 #define pbx_channel_redirecting_effective_to(_a) (_a)->redirecting.to
+#define pbx_channel_redirecting_effective_orig(_a) (_a)->redirecting.orig
+#define pbx_channel_connected_id(_a) (_a)->connected.id
+#define pbx_channel_connected_source(_a) (_a)->connected.source
 
+#if CS_TEST_FRAMEWORK
+#define pbx_test_validate_cleanup ast_test_validate_cleanup
+#define pbx_test_validate ast_test_validate
+#define pbx_test_status_update ast_test_status_update
 #endif
+
 // kate: indent-width 8; replace-tabs off; indent-mode cstyle; auto-insert-doxygen on; line-numbers on; tab-indents on; keep-extra-spaces off; auto-brackets off;
