@@ -8,12 +8,9 @@
  * \note        This program is free software and may be modified and distributed under the terms of the GNU Public License.
  *              See the LICENSE file at the top of the source tree.
  *
- * $Date: 2015-04-07 18:44:04 +0200 (Tue, 07 Apr 2015) $
- * $Revision: 6002 $
  */
-#ifndef __SCCP_CLI_H
-#define __SCCP_CLI_H
-
+#pragma once 
+__BEGIN_EXTERN__
 #define CLI_AMI_LIST_WIDTH 46
 typedef struct sccp_cli_totals {
 	int lines;
@@ -21,6 +18,5 @@ typedef struct sccp_cli_totals {
 } sccp_cli_totals_t;
 void sccp_register_cli(void);
 void sccp_unregister_cli(void);
-
-#endif														// __SCCP_CLI_H
+__END_EXTERN__
 // kate: indent-width 8; replace-tabs off; indent-mode cstyle; auto-insert-doxygen on; line-numbers on; tab-indents on; keep-extra-spaces off; auto-brackets off;
