@@ -8,31 +8,36 @@
  * \note        This program is free software and may be modified and distributed under the terms of the GNU Public License.
  *              See the LICENSE file at the top of the source tree.
  *
- * $Date: 2015-03-14 17:24:31 +0100 (Sat, 14 Mar 2015) $
- * $Revision: 5965 $
  */
+#pragma once
 
-#ifndef _SCCP_DLLISTS_H
-#define _SCCP_DLLISTS_H
+/* Lock Macro for Lists */
+#define SCCP_LIST_LOCK(x)			pbx_mutex_lock(&(x)->lock)
+#define SCCP_LIST_UNLOCK(x)			pbx_mutex_unlock(&(x)->lock)
+#define SCCP_LIST_TRYLOCK(x)			pbx_mutex_trylock(&(x)->lock)
 
-#include "sccp_lock.h"
+/* Lock Macro for read/write Lists */
+#define SCCP_RWLIST_RDLOCK(x)			pbx_rwlock_rdlock(&(x)->lock)
+#define SCCP_RWLIST_WRLOCK(x)			pbx_rwlock_wrlock(&(x)->lock)
+#define SCCP_RWLIST_UNLOCK(x)			pbx_rwlock_unlock(&(x)->lock)
+#define SCCP_RWLIST_TRYRDLOCK(x)		pbx_rwlock_tryrdlock(&(x)->lock)
+#define SCCP_RWLIST_TRYWRLOCK(x)		pbx_rwlock_trywrlock(&(x)->lock)
 
 /* Main list head */
 #define SCCP_LIST_HEAD(name, type)									\
 struct name {												\
+	pbx_mutex_t lock;										\
 	type *first;											\
 	type *last;											\
-	pbx_mutex_t lock;										\
-	uint16_t size;											\
+	uint32_t size;											\
 }
 
 #define SCCP_RWLIST_HEAD(name, type)									\
 struct name {												\
+	pbx_rwlock_t lock;										\
 	type *first;											\
 	type *last;											\
-	uint16_t size;											\
-	uint8_t pad[6];											\
-	pbx_rwlock_t lock;										\
+	uint32_t size;											\
 }
 
 /* Initialize list head */
@@ -374,21 +379,44 @@ struct {												\
 })
 #define SCCP_RWLIST_REMOVE SCCP_LIST_REMOVE
 
-#define SCCP_LIST_FIND(head, var, field, compare, retain) ({						\
-	for((var) = (head)->first; (var); (var) = (var)->field.next) {					\
-	        if ((var) && compare) {									\
-	        	if (retain) {									\
-		                sccp_refcount_retain((var), __FILE__, __LINE__, __PRETTY_FUNCTION__);	\
+/* Expensive SCCP_LIST_FIND version: only used during refcount issue finding */
+/*
+#define SCCP_LIST_FIND(_head, _type, _var, _field, _compare, _retain, _file, _line, _func) ({		\
+	_type *_var;											\
+	_type *__tmp_##_var##_line;									\
+	for((_var) = (_head)->first; (_var); (_var) = ((_var) ? (_var)->_field.next : NULL)) {		\
+		__tmp_##_var##_line = sccp_refcount_retain((_var), _file, _line, _func);		\
+	        if (__tmp_##_var##_line) {								\
+	        	if (_compare) {									\
+	        		if (!_retain) {								\
+		 		        sccp_refcount_release(__tmp_##_var##_line, _file, _line, _func);\
+	        		}									\
+				break;									\
+		        }										\
+ 		        sccp_refcount_release(__tmp_##_var##_line, _file, _line, _func);		\
+		} else {										\
+			pbx_log(LOG_ERROR, "SCCP (%s:%d:%s): Failed to get reference to variable during SCCP_LIST_FIND\n", _file, _line, _func);\
+			(_var) = NULL;									\
+		}											\
+	}                                                                                               \
+	(_var);												\
+})
+*/
+#define SCCP_LIST_FIND(_head, _type, _var, _field, _compare, _retain, _file, _line, _func) ({		\
+	_type *_var;											\
+	for((_var) = (_head)->first; (_var); (_var) = ((_var) ? (_var)->_field.next : NULL)) {		\
+		if (_compare) {										\
+			if (_retain) {									\
+				sccp_refcount_retain((_var), _file, _line, _func);			\
 			}										\
 			break;										\
-	        }											\
+		}											\
 	}                                                                                               \
-	(var);												\
+	(_var);												\
 })
 
 #define SCCP_RWLIST_FIND SCCP_LIST_FIND
 
 #define SCCP_LIST_GETSIZE(head) (head)->size
 #define SCCP_RWLIST_GETSIZE SCCP_LIST_GETSIZE
-#endif														/* _SCCP_DLLISTS_H */
 // kate: indent-width 8; replace-tabs off; indent-mode cstyle; auto-insert-doxygen on; line-numbers on; tab-indents on; keep-extra-spaces off; auto-brackets off;
