@@ -6,8 +6,6 @@
  *              See the LICENSE file at the top of the source tree.
  * \since       2009-06-15
  *
- * $Date: 2014-11-05 16:40:01 +0100 (Wed, 05 Nov 2014) $
- * $Revision: 5790 $
  */
 
 /*!
@@ -25,27 +23,29 @@
 #include "sccp_device.h"
 #include "sccp_line.h"
 #include "sccp_utils.h"
-#include "sccp_mwi.h"
 
 #ifdef CS_DEVSTATE_FEATURE
 #include "sccp_devstate.h"
 #endif
 
-SCCP_FILE_VERSION(__FILE__, "$Revision: 5790 $")
+SCCP_FILE_VERSION(__FILE__, "");
 
-    /*!
-     * \brief Feature Button Changed
-     *
-     * fetch the new state, and send status to device
-     *
-     * \param device SCCP Device
-     * \param featureType SCCP Feature Type
-     * 
-     * \warning
-     *  - device->buttonconfig is not always locked
-     *
-     */
-void sccp_featButton_changed(sccp_device_t * device, sccp_feature_type_t featureType)
+#if defined(CS_AST_HAS_EVENT) && defined(HAVE_PBX_EVENT_H) 	// ast_event_subscribe
+#  include <asterisk/event.h>
+#endif
+
+/*!
+ * \brief Feature Button Changed
+ *
+ * fetch the new state, and send status to device
+ *
+ * \param device SCCP Device
+ * \param featureType SCCP Feature Type
+ * 
+ * \warning
+ *  - device->buttonconfig is not always locked
+ */
+void sccp_featButton_changed(constDevicePtr device, sccp_feature_type_t featureType)
 {
 	sccp_msg_t *msg = NULL;
 	sccp_buttonconfig_t *config = NULL, *buttonconfig = NULL;
@@ -57,10 +57,10 @@ void sccp_featButton_changed(sccp_device_t * device, sccp_feature_type_t feature
 		return;
 	}
 
-	SCCP_LIST_LOCK(&device->buttonconfig);
+	SCCP_LIST_LOCK(&((devicePtr)device)->buttonconfig);
 	SCCP_LIST_TRAVERSE(&device->buttonconfig, config, list) {
 		if (config->type == FEATURE && config->button.feature.id == featureType) {
-			sccp_log((DEBUGCAT_FEATURE_BUTTON + DEBUGCAT_FEATURE)) (VERBOSE_PREFIX_3 "%s: FeatureID = %d, Option: %s\n", DEV_ID_LOG(device), config->button.feature.id, (config->button.feature.options) ? config->button.feature.options : "(none)");
+			sccp_log((DEBUGCAT_FEATURE_BUTTON + DEBUGCAT_FEATURE)) (VERBOSE_PREFIX_3 "%s: (sccp_featButton_changed) FeatureID = %d, Option: %s\n", DEV_ID_LOG(device), config->button.feature.id, (config->button.feature.options) ? config->button.feature.options : "(none)");
 			instance = config->instance;
 
 			switch (config->button.feature.id) {
@@ -70,13 +70,13 @@ void sccp_featButton_changed(sccp_device_t * device, sccp_feature_type_t feature
 					}
 
 					sccp_log((DEBUGCAT_FEATURE_BUTTON + DEBUGCAT_FEATURE)) (VERBOSE_PREFIX_3 "%s: device->privacyFeature.status=%d\n", DEV_ID_LOG(device), device->privacyFeature.status);
-					if (!strcasecmp(config->button.feature.options, "callpresent")) {
+					if (sccp_strcaseequals(config->button.feature.options, "callpresent")) {
 						uint32_t result = device->privacyFeature.status & SCCP_PRIVACYFEATURE_CALLPRESENT;
 
 						sccp_log((DEBUGCAT_FEATURE_BUTTON + DEBUGCAT_FEATURE)) (VERBOSE_PREFIX_3 "%s: result is %d\n", device->id, result);
 						config->button.feature.status = (result) ? 1 : 0;
 					}
-					if (!strcasecmp(config->button.feature.options, "hint")) {
+					if (sccp_strcaseequals(config->button.feature.options, "hint")) {
 						uint32_t result = device->privacyFeature.status & SCCP_PRIVACYFEATURE_HINT;
 
 						sccp_log((DEBUGCAT_FEATURE_BUTTON + DEBUGCAT_FEATURE)) (VERBOSE_PREFIX_3 "%s: result is %d\n", device->id, result);
@@ -119,13 +119,13 @@ void sccp_featButton_changed(sccp_device_t * device, sccp_feature_type_t feature
 					break;
 
 				case SCCP_FEATURE_DND:
-					if (!strcasecmp(config->button.feature.options, "silent")) {
+					if (sccp_strcaseequals(config->button.feature.options, "silent")) {
 						if ((device->dndFeature.enabled && device->dndFeature.status == SCCP_DNDMODE_SILENT)) {
 							config->button.feature.status = 1;
 						} else {
 							config->button.feature.status = 0;
 						}
-					} else if (!strcasecmp(config->button.feature.options, "busy")) {
+					} else if (sccp_strcaseequals(config->button.feature.options, "busy")) {
 						if ((device->dndFeature.enabled && device->dndFeature.status == SCCP_DNDMODE_REJECT)) {
 							config->button.feature.status = 1;
 						} else {
@@ -134,7 +134,7 @@ void sccp_featButton_changed(sccp_device_t * device, sccp_feature_type_t feature
 					}
 					break;
 				case SCCP_FEATURE_MONITOR:
-					sccp_log((DEBUGCAT_FEATURE_BUTTON)) (VERBOSE_PREFIX_3 "%s: monitor feature state: %d\n", DEV_ID_LOG(device), device->monitorFeature.status);
+					sccp_log((DEBUGCAT_FEATURE_BUTTON)) (VERBOSE_PREFIX_3 "%s: (sccp_featButton_changed) monitor featureButton new state:%s (%d)\n", DEV_ID_LOG(device), sccp_feature_monitor_state2str(device->monitorFeature.status), device->monitorFeature.status);
 					if (device->inuseprotocolversion > 15) {				// multiple States
 						buttonID = SKINNY_BUTTONTYPE_MULTIBLINKFEATURE;
 						switch (device->monitorFeature.status) {
@@ -142,17 +142,16 @@ void sccp_featButton_changed(sccp_device_t * device, sccp_feature_type_t feature
 								config->button.feature.status = 0;
 								break;
 							case SCCP_FEATURE_MONITOR_STATE_REQUESTED:
-								config->button.feature.status = 131586;		// old BUTTONTYPE_FEATURE interprets this a simple TRUE value
+								config->button.feature.status = 0x020202;
 								break;
 							case SCCP_FEATURE_MONITOR_STATE_ACTIVE:
-								config->button.feature.status = 131843;
+								config->button.feature.status = 0x020303;
 								break;
 							case (SCCP_FEATURE_MONITOR_STATE_REQUESTED | SCCP_FEATURE_MONITOR_STATE_ACTIVE):
-								config->button.feature.status = 131589;
+								config->button.feature.status = 0x020205;
 								break;
 						}
 					} else {
-						sccp_log(DEBUGCAT_FEATURE_BUTTON + DEBUGCAT_FEATURE) (VERBOSE_PREFIX_3 "%s: (featButton_changed) state %d\n", device->id, device->monitorFeature.status);
 						switch (device->monitorFeature.status) {
 							case SCCP_FEATURE_MONITOR_STATE_DISABLED:
 								config->button.feature.status = 0;
@@ -283,22 +282,22 @@ void sccp_featButton_changed(sccp_device_t * device, sccp_feature_type_t feature
 			/* send status using new message */
 			if (device->inuseprotocolversion >= 15) {
 				REQ(msg, FeatureStatDynamicMessage);
-				msg->data.FeatureStatDynamicMessage.lel_instance = htolel(instance);
-				msg->data.FeatureStatDynamicMessage.lel_type = htolel(buttonID);
-				msg->data.FeatureStatDynamicMessage.lel_status = htolel(config->button.feature.status);
-				sccp_copy_string(msg->data.FeatureStatDynamicMessage.DisplayName, config->label, strlen(config->label) + 1);
+				msg->data.FeatureStatDynamicMessage.lel_featureIndex = htolel(instance);
+				msg->data.FeatureStatDynamicMessage.lel_featureID = htolel(buttonID);
+				msg->data.FeatureStatDynamicMessage.lel_featureStatus = htolel(config->button.feature.status);
+				sccp_copy_string(msg->data.FeatureStatDynamicMessage.featureTextLabel, config->label, sizeof(msg->data.FeatureStatDynamicMessage.featureTextLabel));
 			} else {
 				REQ(msg, FeatureStatMessage);
-				msg->data.FeatureStatMessage.lel_featureInstance = htolel(instance);
+				msg->data.FeatureStatMessage.lel_featureIndex = htolel(instance);
 				msg->data.FeatureStatMessage.lel_featureID = htolel(buttonID);
 				msg->data.FeatureStatMessage.lel_featureStatus = htolel(config->button.feature.status);
-				sccp_copy_string(msg->data.FeatureStatMessage.featureTextLabel, config->label, strlen(config->label) + 1);
+				sccp_copy_string(msg->data.FeatureStatMessage.featureTextLabel, config->label, sizeof(msg->data.FeatureStatDynamicMessage.featureTextLabel));
 			}
 			sccp_dev_send(device, msg);
-			sccp_log((DEBUGCAT_FEATURE_BUTTON + DEBUGCAT_FEATURE)) (VERBOSE_PREFIX_3 "%s: (sccp_featButton_changed) Got Feature Status Request. Instance = %d, Label: '%s', Status: %d, Nota bene: Config pointer: %p\n", DEV_ID_LOG(device), instance, config->label, config->button.feature.status, config);
+			sccp_log((DEBUGCAT_FEATURE_BUTTON + DEBUGCAT_FEATURE)) (VERBOSE_PREFIX_3 "%s: (sccp_featButton_changed) Got Feature Status Request. Instance = %d, Label: '%s', Status: %d\n", DEV_ID_LOG(device), instance, config->label, config->button.feature.status);
 		}
 	}
-	SCCP_LIST_UNLOCK(&device->buttonconfig);
+	SCCP_LIST_UNLOCK(&((devicePtr)device)->buttonconfig);
 }
 
 /*!
