@@ -4,9 +4,6 @@
  * \author      Marcello Ceschia <marcello [at] ceschia.de>
  * \note        This program is free software and may be modified and distributed under the terms of the GNU Public License.
  *              See the LICENSE file at the top of the source tree.
- * 
- * $Date: 2015-03-18 12:16:11 +0100 (Wed, 18 Mar 2015) $
- * $Revision: 5979 $  
  */
 #include <config.h>
 #ifdef CS_SCCP_MANAGER
@@ -20,8 +17,9 @@
 #include "sccp_features.h"
 #include "sccp_actions.h"
 #include "sccp_socket.h"
+#include <asterisk/threadstorage.h>
 
-SCCP_FILE_VERSION(__FILE__, "$Revision: 5979 $");
+SCCP_FILE_VERSION(__FILE__, "");
 
 /*
  * Descriptions
@@ -31,7 +29,7 @@ static char management_show_lines_desc[] = "Description: Lists SCCP lines in tex
 static char management_restart_devices_desc[] = "Description: restart a given device\n" "\n" "Variables:\n" "   Devicename: Name of device to restart\n";
 static char management_show_device_add_line_desc[] = "Description: Lists SCCP devices in text format with details on current status.\n" "\n" "DevicelistComplete.\n" "Variables: \n" "  Devicename: Name of device to restart.\n" "  Linename: Name of line";
 static char management_device_update_desc[] = "Description: restart a given device\n" "\n" "Variables:\n" "   Devicename: Name of device\n";
-static char management_device_set_dnd_desc[] = "Description: set dnd on device\n" "\n" "Variables:\n" "   Devicename: Name of device\n" "  DNDState: on (busy) / off / reject/ silent";
+static char management_device_set_dnd_desc[] = "Description: set dnd on device\n" "\n" "Variables:\n" "   Devicename: Name of device\n" "  DNDState: off / reject / silent";
 static char management_line_fwd_update_desc[] = "Description: update forward status for line\n" "\n" "Variables:\n" "  Devicename: Name of device\n" "  Linename: Name of line\n" "  Forwardtype: type of cfwd (all | busy | noAnswer)\n" "  Disable: yes Disable call forward (optional)\n" "  Number: number to forward calls (optional)";
 static char management_fetch_config_metadata_desc[] = "Description: fetch configuration metadata\n" "\n" "Variables:\n" "  segment: Config Segment Name (if empty returns all segments).\n";
 
@@ -186,7 +184,7 @@ void sccp_manager_eventListener(const sccp_event_t * event)
 			manager_event(EVENT_FLAG_CALL,
 				      "PeerStatus",
 				      "ChannelType: SCCP\r\nChannelObjectType: DeviceLine\r\nPeerStatus: %s\r\nSCCPDevice: %s\r\nSCCPLine: %s\r\nSCCPLineName: %s\r\nSubscriptionId: %s\r\nSubscriptionName: %s\r\n",
-				      "ATTACHED", DEV_ID_LOG(device), linedevice && linedevice->line ? linedevice->line->name : "(null)", linedevice && linedevice->line ? linedevice->line->label : "(null)", linedevice->subscriptionId.number ? linedevice->subscriptionId.number : "(null)", linedevice->subscriptionId.name ? linedevice->subscriptionId.name : "(null)");
+				      "ATTACHED", DEV_ID_LOG(device), linedevice && linedevice->line ? linedevice->line->name : "(null)", (linedevice && linedevice->line && linedevice->line->label) ? linedevice->line->label : "(null)", linedevice->subscriptionId.number, linedevice->subscriptionId.name);
 			break;
 
 		case SCCP_EVENT_DEVICE_DETACHED:
@@ -195,7 +193,7 @@ void sccp_manager_eventListener(const sccp_event_t * event)
 			manager_event(EVENT_FLAG_CALL,
 				      "PeerStatus",
 				      "ChannelType: SCCP\r\nChannelObjectType: DeviceLine\r\nPeerStatus: %s\r\nSCCPDevice: %s\r\nSCCPLine: %s\r\nSCCPLineName: %s\r\nSubscriptionId: %s\r\nSubscriptionName: %s\r\n",
-				      "DETACHED", DEV_ID_LOG(device), linedevice && linedevice->line ? linedevice->line->name : "(null)", linedevice && linedevice->line ? linedevice->line->label : "(null)", linedevice->subscriptionId.number ? linedevice->subscriptionId.number : "(null)", linedevice->subscriptionId.name ? linedevice->subscriptionId.name : "(null)");
+				      "DETACHED", DEV_ID_LOG(device), linedevice && linedevice->line ? linedevice->line->name : "(null)", (linedevice && linedevice->line && linedevice->line->label) ? linedevice->line->label : "(null)", linedevice->subscriptionId.number, linedevice->subscriptionId.name);
 			break;
 
 		case SCCP_EVENT_FEATURE_CHANGED:
@@ -205,7 +203,7 @@ void sccp_manager_eventListener(const sccp_event_t * event)
 
 			switch (featureType) {
 				case SCCP_FEATURE_DND:
-					manager_event(EVENT_FLAG_CALL, "DND", "ChannelType: SCCP\r\nChannelObjectType: Device\r\nFeature: %s\r\nStatus: %s\r\nSCCPDevice: %s\r\n", featureType2str(SCCP_FEATURE_DND), sccp_dndmode2str(device->dndFeature.status), DEV_ID_LOG(device));
+					manager_event(EVENT_FLAG_CALL, "DND", "ChannelType: SCCP\r\nChannelObjectType: Device\r\nFeature: %s\r\nStatus: %s\r\nSCCPDevice: %s\r\n", sccp_feature_type2str(SCCP_FEATURE_DND), sccp_dndmode2str(device->dndFeature.status), DEV_ID_LOG(device));
 					break;
 				case SCCP_FEATURE_CFWDALL:
 				case SCCP_FEATURE_CFWDBUSY:
@@ -213,12 +211,12 @@ void sccp_manager_eventListener(const sccp_event_t * event)
 						manager_event(EVENT_FLAG_CALL,
 							      "CallForward",
 							      "ChannelType: SCCP\r\nChannelObjectType: DeviceLine\r\nFeature: %s\r\nStatus: %s\r\nExtension: %s\r\nSCCPLine: %s\r\nSCCPDevice: %s\r\n",
-							      featureType2str(featureType), (SCCP_FEATURE_CFWDALL == featureType) ? ((linedevice->cfwdAll.enabled) ? "On" : "Off") : ((linedevice->cfwdBusy.enabled) ? "On" : "Off"), (SCCP_FEATURE_CFWDALL == featureType) ? ((linedevice->cfwdAll.number) ? linedevice->cfwdAll.number : "(null)") : ((linedevice->cfwdBusy.number) ? linedevice->cfwdBusy.number : "(null)"), (linedevice->line) ? linedevice->line->name : "(null)", DEV_ID_LOG(device)
+							      sccp_feature_type2str(featureType), (SCCP_FEATURE_CFWDALL == featureType) ? ((linedevice->cfwdAll.enabled) ? "On" : "Off") : ((linedevice->cfwdBusy.enabled) ? "On" : "Off"), (SCCP_FEATURE_CFWDALL == featureType) ? linedevice->cfwdAll.number : linedevice->cfwdBusy.number, (linedevice->line) ? linedevice->line->name : "(null)", DEV_ID_LOG(device)
 						    );
 					}
 					break;
 				case SCCP_FEATURE_CFWDNONE:
-					manager_event(EVENT_FLAG_CALL, "CallForward", "ChannelType: SCCP\r\nChannelObjectType: DeviceLine\r\nFeature: %s\r\nStatus: Off\r\nSCCPLine: %s\r\nSCCPDevice: %s\r\n", featureType2str(featureType), (linedevice && linedevice->line) ? linedevice->line->name : "(null)", DEV_ID_LOG(device));
+					manager_event(EVENT_FLAG_CALL, "CallForward", "ChannelType: SCCP\r\nChannelObjectType: DeviceLine\r\nFeature: %s\r\nStatus: Off\r\nSCCPLine: %s\r\nSCCPDevice: %s\r\n", sccp_feature_type2str(featureType), (linedevice && linedevice->line) ? linedevice->line->name : "(null)", DEV_ID_LOG(device));
 					break;
 				default:
 					break;
@@ -258,8 +256,9 @@ static int sccp_manager_show_devices(struct mansession *s, const struct message
 	SCCP_RWLIST_TRAVERSE(&GLOB(devices), device, list) {
 		timeinfo = localtime(&device->registrationTime);
 
-		if (device->session) {
-			sccp_copy_string(clientAddress, sccp_socket_stringify(&device->session->sin), sizeof(clientAddress));
+		struct sockaddr_storage sas = { 0 };
+		if (sccp_session_getSas(device->session, &sas)) {
+			sccp_copy_string(clientAddress, sccp_socket_stringify(&sas), sizeof(clientAddress));
 		} else {
 			sccp_copy_string(clientAddress, "--", sizeof(clientAddress));
 		}
@@ -269,9 +268,9 @@ static int sccp_manager_show_devices(struct mansession *s, const struct message
 		astman_append(s, "ChannelType: SCCP\r\n");
 		astman_append(s, "ObjectId: %s\r\n", device->id);
 		astman_append(s, "ObjectType: device\r\n");
-		astman_append(s, "Description: %s\r\n", device->description);
+		astman_append(s, "Description: %s\r\n", device->description  ? device->description : "<not set>");
 		astman_append(s, "IPaddress: %s\r\n", clientAddress);
-		astman_append(s, "Reg_Status: %s\r\n", skinny_registrationstate2str(device->registrationState));
+		astman_append(s, "Reg_Status: %s\r\n", skinny_registrationstate2str(sccp_device_getRegistrationState(device)));
 		astman_append(s, "Reg_Time: %s\r\n", regtime);
 		astman_append(s, "Active: %s\r\n", (device->active_channel) ? "Yes" : "No");
 		astman_append(s, "NumLines: %d\r\n\r\n", device->configurationStatistic.numberOfLines);
@@ -299,7 +298,7 @@ static int sccp_manager_show_devices(struct mansession *s, const struct message
 static int sccp_manager_show_lines(struct mansession *s, const struct message *m)
 {
 	const char *id = astman_get_header(m, "ActionID");
-	sccp_line_t *line;
+	sccp_line_t *line = NULL;
 	char idtext[256] = "";
 	int total = 0;
 
@@ -314,7 +313,7 @@ static int sccp_manager_show_lines(struct mansession *s, const struct message *m
 		astman_append(s, "ObjectId: %s\r\n", line->id);
 		astman_append(s, "ObjectType: line\r\n");
 		astman_append(s, "Name: %s\r\n", line->name);
-		astman_append(s, "Description: %s\r\n", line->description);
+		astman_append(s, "Description: %s\r\n", line->description  ? line->description : "<not set>");
 		astman_append(s, "Num_Channels: %d\r\n\r\n", SCCP_RWLIST_GETSIZE(&line->channels));
 		total++;
 	}
@@ -415,9 +414,15 @@ static int sccp_manager_device_add_line(struct mansession *s, const struct messa
 		astman_send_error(s, m, "Line not found");
 		return 0;
 	}
-	sccp_config_addButton(d, -1, LINE, line->name, NULL, NULL);
-	astman_append(s, "Done\r\n");
-	astman_append(s, "\r\n");
+	if (sccp_config_addButton(&d->buttonconfig, -1, LINE, line->name, NULL, NULL) == SCCP_CONFIG_CHANGE_CHANGED) {
+		d->pendingUpdate = 1;
+		sccp_config_addButton(&d->buttonconfig, -1, LINE, line->name, NULL, NULL);
+		sccp_device_check_update(d);
+		astman_append(s, "Done\r\n");
+		astman_append(s, "\r\n");
+	} else {
+		astman_send_error(s, m, "Adding line button to device failed");
+	}
 	return 0;
 }
 
@@ -576,7 +581,7 @@ static int sccp_manager_device_set_dnd(struct mansession *s, const struct messag
 {
 	const char *deviceName = astman_get_header(m, "Devicename");
 	const char *DNDState = astman_get_header(m, "DNDState");
-	int prevStatus = 0;
+	uint prevStatus = 0;
 	char retValStr[64] = "";
 
 	/** we need the device for resuming calls */
@@ -594,7 +599,7 @@ static int sccp_manager_device_set_dnd(struct mansession *s, const struct messag
 	if (d) {
 		if (d->dndFeature.enabled) {
 			prevStatus = d->dndFeature.status;
-			if (sccp_strcaseequals("on", DNDState) || sccp_strcaseequals("reject", DNDState)) {
+			if (sccp_strcaseequals("reject", DNDState)) {
 				d->dndFeature.status = SCCP_DNDMODE_REJECT;
 			} else if (sccp_strcaseequals("silent", DNDState)) {
 				d->dndFeature.status = SCCP_DNDMODE_SILENT;
@@ -650,7 +655,7 @@ static int sccp_manager_startCall(struct mansession *s, const struct message *m)
 		if (d && d->defaultLineInstance > 0) {
 			line = sccp_line_find_byid(d, d->defaultLineInstance);
 		} else {
-			line = sccp_dev_get_activeline(d);
+			line = sccp_dev_getActiveLine(d);
 		}
 	} else {
 		line = sccp_line_find_byname(lineName, FALSE);
@@ -668,8 +673,8 @@ static int sccp_manager_startCall(struct mansession *s, const struct message *m)
 		.uniqueid = astman_get_header(m, "ChannelId"),
 		//.uniqueid2 = astman_get_header(m, "OtherChannelId")
 	};
-	if ((ids.uniqueid && AST_MAX_PUBLIC_UNIQUEID < strlen(ids.uniqueid))
-	    //|| (ids.uniqueid2 && AST_MAX_PUBLIC_UNIQUEID < strlen(ids.uniqueid2))
+	if ((ids.uniqueid && AST_MAX_PUBLIC_UNIQUEID < sccp_strlen(ids.uniqueid))
+	    //|| (ids.uniqueid2 && AST_MAX_PUBLIC_UNIQUEID < sccp_strlen(ids.uniqueid2))
 	    ) {
 		astman_send_error_va(s, m, "Uniqueid length exceeds maximum of %d\n", AST_MAX_PUBLIC_UNIQUEID);
 		return 0;
@@ -717,7 +722,7 @@ static int sccp_manager_answerCall2(struct mansession *s, const struct message *
 			if (c->state == SCCP_CHANNELSTATE_RINGING) {
 				sccp_channel_answer(d, c);
 				if (c->owner) {
-					PBX(queue_control) (c->owner, AST_CONTROL_ANSWER);
+					iPbx.queue_control(c->owner, AST_CONTROL_ANSWER);
 				}
 				snprintf(retValStr, sizeof(retValStr), "Answered channel '%s' on device '%s'\r\n", channelId, deviceName);
 				astman_send_ack(s, m, retValStr);
@@ -839,7 +844,7 @@ static void sccp_asterisk_parseStrToAstMessage(char *str, struct message *m)
 	int x = 0;
 	int curlen;
 
-	curlen = strlen(str);
+	curlen = sccp_strlen(str);
 	for (x = 0; x < curlen; x++) {
 		int cr;												/* set if we have \r */
 
@@ -870,48 +875,53 @@ static void sccp_asterisk_parseStrToAstMessage(char *str, struct message *m)
  */
 static int sccp_asterisk_managerHookHelper(int category, const char *event, char *content)
 {
-	PBX_CHANNEL_TYPE *pbxchannel = NULL;
-	PBX_CHANNEL_TYPE *pbxBridge = NULL;
 	char *str, *dupStr;
 
 	if (EVENT_FLAG_CALL == category) {
 		if (!strcasecmp("MonitorStart", event) || !strcasecmp("MonitorStop", event)) {
+			AUTO_RELEASE sccp_channel_t *channel = NULL;
 			struct message m = { 0 };
 
-			str = dupStr = sccp_strdupa(content); /** need a dup, because converter to message structure will modify the str */
+			str = dupStr = pbx_strdupa(content); /** need a dup, because converter to message structure will modify the str */
+			sccp_log(DEBUGCAT_CORE)("SCCP: (managerHookHelper) MonitorStart/MonitorStop Received\ncontent:[%s]\n", content);	/* temp */
 
 			sccp_asterisk_parseStrToAstMessage(str, &m); /** convert to message structure to use the astman_get_header function */
 			const char *channelName = astman_get_header(&m, "Channel");
 
-			pbxchannel = pbx_channel_get_by_name(channelName);
+			PBX_CHANNEL_TYPE *pbxchannel = pbx_channel_get_by_name(channelName);							/* returns reffed */
+			if (pbxchannel) {
+				PBX_CHANNEL_TYPE *pbxBridge = NULL;
+				if ((CS_AST_CHANNEL_PVT_IS_SCCP(pbxchannel))) {
+					channel = get_sccp_channel_from_pbx_channel(pbxchannel);
+				} else if ( (pbxBridge = pbx_channel_get_by_name(pbx_builtin_getvar_helper(pbxchannel, "BRIDGEPEER"))) ) {
+					if ((CS_AST_CHANNEL_PVT_IS_SCCP(pbxBridge))) {
+						channel = get_sccp_channel_from_pbx_channel(pbxBridge);
+					}
 #if ASTERISK_VERSION_GROUP == 106
-			pbx_channel_unlock(pbxchannel);
+					pbx_channel_unlock(pbxBridge);
+#else
+					pbxBridge = ast_channel_unref(pbxBridge);
 #endif
-
-			AUTO_RELEASE sccp_channel_t *channel = NULL;
-
-			if (pbxchannel && (CS_AST_CHANNEL_PVT_IS_SCCP(pbxchannel))) {
-				channel = get_sccp_channel_from_pbx_channel(pbxchannel);
-			} else if (pbxchannel && ((pbxBridge = pbx_channel_get_by_name(pbx_builtin_getvar_helper(pbxchannel, "BRIDGEPEER"))) != NULL) && (CS_AST_CHANNEL_PVT_IS_SCCP(pbxBridge))) {
-				channel = get_sccp_channel_from_pbx_channel(pbxBridge);
+				}
 #if ASTERISK_VERSION_GROUP == 106
-				pbx_channel_unlock(pbxBridge);
+				pbx_channel_unlock(pbxchannel);
+#else
+				pbxchannel = ast_channel_unref(pbxchannel);
 #endif
 			}
 
 			if (channel) {
+				sccp_log(DEBUGCAT_CORE)("%s: (managerHookHelper) MonitorStart/MonitorStop Received\n", channel->designator);	/* temp */
 				AUTO_RELEASE sccp_device_t *d = sccp_channel_getDevice_retained(channel);
-
 				if (d) {
+					sccp_log(DEBUGCAT_CORE)("%s: (managerHookHelper) MonitorStart/MonitorStop on Device: %s\n", channel->designator, d->id);	/* temp */
 					if (!strcasecmp("MonitorStart", event)) {
 						d->monitorFeature.status |= SCCP_FEATURE_MONITOR_STATE_ACTIVE;
 					} else {
 						d->monitorFeature.status &= ~SCCP_FEATURE_MONITOR_STATE_ACTIVE;
 					}
 					sccp_msg_t *msg_out = NULL;
-
 					REQ(msg_out, RecordingStatusMessage);
-
 					msg_out->data.RecordingStatusMessage.lel_callReference = htolel(channel->callid);
 					msg_out->data.RecordingStatusMessage.lel_status = (d->monitorFeature.status & SCCP_FEATURE_MONITOR_STATE_ACTIVE) ? htolel(1) : htolel(0);
 					sccp_dev_send(d, msg_out);
@@ -923,6 +933,78 @@ static int sccp_asterisk_managerHookHelper(int category, const char *event, char
 	}
 	return 0;
 }
+
+AST_THREADSTORAGE(hookresult_threadbuf);
+#define HOOKRESULT_INITSIZE   DEFAULT_PBX_STR_BUFFERSIZE
+
+#if ASTERISK_VERSION_GROUP >= 108
+/*
+ * \brief helper function to concatenate the result from a ami hook send action using a threadlocal buffer
+ */
+static int __sccp_manager_hookresult(int category, const char *event, char *content) {
+        struct ast_str *buf = ast_str_thread_get(&hookresult_threadbuf, HOOKRESULT_INITSIZE);;
+	if (buf) {
+		pbx_str_append(&buf, 0, "%s", content);
+	}
+	return 0;
+}
+#endif
+/*!
+ * \brief Call an AMI/Manager Function and Wait for the Result
+ * 
+ * @param manager_command	const char * containing Something like "Action: ParkedCalls\r\n"
+ * @param outStr		unallocated char * (will be allocated if successfull, must be freed after call)
+ * @return int (-1 on failure | return value from called function)
+ *
+ * \todo implement using thread_local instead
+ */
+boolean_t sccp_manager_action2str(const char *manager_command, char **outStr) 
+{
+#if ASTERISK_VERSION_GROUP >= 108
+        int failure = 0;
+        struct ast_str *buf;
+        
+        if (!outStr || sccp_strlen_zero(manager_command) || !(buf = ast_str_thread_get(&hookresult_threadbuf, HOOKRESULT_INITSIZE))) {
+		pbx_log(LOG_ERROR, "SCCP: No OutStr or Command Provided\n");
+        	return -2;
+        }
+
+	struct manager_custom_hook hook = {__FILE__, __sccp_manager_hookresult};
+        failure = ast_hook_send_action(&hook, manager_command);							/* "Action: ParkedCalls\r\n" */
+        if (!failure) {
+		sccp_log(DEBUGCAT_CORE)("SCCP: Sending AMI Result String: %s\n", pbx_str_buffer(buf));
+        	*outStr = pbx_strdup(pbx_str_buffer(buf));
+        }
+       	ast_str_reset(buf);
+        return !failure ? TRUE : FALSE;
+#else
+	sccp_log(DEBUGCAT_CORE)("SCCP: ast_hook_send_action is not available in asterisk-1.6\n");
+	return FALSE;
+#endif
+}
+
+/* example use
+void example_function() {
+	char *outStr;
+	char *manager_command = "Action: ParkedCalls\r\n";
+	if (sccp_manager_action2str(manager_command, &outStr) >= 0 && outStr) {
+		sccp_log(DEBUGCAT_CORE)("SCCP: hook result: %s\n", outStr);
+		sccp_free(outStr);
+	}
+
+	struct ast_str *amiCommandStr = ast_str_alloca(DEFAULT_PBX_STR_BUFFERSIZE);
+	pbx_str_append(&amiCommandStr,0 ,"Action: MixMonitor\r\n");
+	pbx_str_append(&amiCommandStr,0 ,"Channel: SCCP/98031-00000001\r\n");
+	pbx_str_append(&amiCommandStr,0 ,"File: mixmonitor-test.wav\r\n");
+	pbx_str_append(&amiCommandStr,0 ,"Options: ab\r\n\r\n");
+	
+	if (sccp_manager_action2str(pbx_str_buffer(amiCommandStr), &outStr) >= 0 && outStr) {
+		sccp_log(DEBUGCAT_CORE)("SCCP: hook result: %s\n", outStr);
+		sccp_free(outStr);
+	}
+}
+*/
+
 #endif														/* HAVE_PBX_MANAGER_HOOK_H */
 
 #endif														/* CS_SCCP_MANAGER */
