@@ -7,13 +7,8 @@
  *              Modified by Jan Czmok and Julien Goodwin
  * \note        This program is free software and may be modified and distributed under the terms of the GNU Public License.
  *              See the LICENSE file at the top of the source tree.
- *
- * $Date: 2014-11-05 16:40:01 +0100 (Wed, 05 Nov 2014) $
- * $Revision: 5790 $  
  */
-#ifndef __SCCP_PBX_H
-#define __SCCP_PBX_H
-
+#pragma once
 uint8_t sccp_pbx_channel_allocate(sccp_channel_t * c, const void *ids, const PBX_CHANNEL_TYPE * parentChannel);
 int sccp_pbx_sched_dial(const void *data);
 sccp_extension_status_t sccp_pbx_helper(sccp_channel_t * c);
@@ -32,5 +27,4 @@ int sccp_pbx_answer(sccp_channel_t * c);
 // It is a function so we can intervene in the standard asterisk bridge method. 
 // At this moment it provides a little logging and switches a couple of DTMF signals off when bridging SCCP<->SCCP calls - DdG
 //! \todo move this to pbx impl
-#endif
 // kate: indent-width 8; replace-tabs off; indent-mode cstyle; auto-insert-doxygen on; line-numbers on; tab-indents on; keep-extra-spaces off; auto-brackets off;
