@@ -8,15 +8,12 @@
  *              Library providing a threading pool where you can add work. 
  * \since       2009-01-16
  *
- * $Date: 2015-01-14 13:45:24 +0100 (Wed, 14 Jan 2015) $
- * $Revision: 5835 $
  */
 
 #include <config.h>
 #include "common.h"
-#include "sccp_threadpool.h"
 
-SCCP_FILE_VERSION(__FILE__, "$Revision: 5835 $");
+SCCP_FILE_VERSION(__FILE__, "");
 #include "sccp_threadpool.h"
 #include <signal.h>
 #undef pthread_create
@@ -32,9 +29,9 @@ typedef struct sccp_threadpool_thread sccp_threadpool_thread_t;
 
 struct sccp_threadpool_thread {
 	pthread_t thread;
-	int die;
 	sccp_threadpool_t *tp_p;
 	SCCP_LIST_ENTRY (sccp_threadpool_thread_t) list;
+	boolean_t die;
 };
 
 /* The threadpool */
@@ -46,6 +43,7 @@ struct sccp_threadpool {
 	time_t last_size_check;											/*!< Time since last size check */
 	time_t last_resize;											/*!< Time since last resize */
 	int job_high_water_mark;										/*!< Highest number of jobs outstanding since last resize check */
+	volatile int sccp_threadpool_shuttingdown;
 };
 
 /* 
@@ -58,7 +56,6 @@ struct sccp_threadpool {
  * sem                  = semaphore
  * xN                   = x can be any string. N stands for amount
  * */
-static volatile int sccp_threadpool_shuttingdown = 0;
 
 /* Initialise thread pool */
 sccp_threadpool_t *sccp_threadpool_init(int threadsN)
@@ -76,9 +73,8 @@ sccp_threadpool_t *sccp_threadpool_init(int threadsN)
 		threadsN = THREADPOOL_MAX_SIZE;
 	}
 	/* Make new thread pool */
-	tp_p = (sccp_threadpool_t *) sccp_malloc(sizeof(sccp_threadpool_t));					/* MALLOC thread pool */
-	if (tp_p == NULL) {
-		pbx_log(LOG_ERROR, "sccp_threadpool_init(): Could not allocate memory for thread pool\n");
+	if (!(tp_p = sccp_calloc(sizeof *tp_p, 1))) {
+		pbx_log(LOG_ERROR, SS_Memory_Allocation_Error, "SCCP");
 		return NULL;
 	}
 
@@ -90,6 +86,7 @@ sccp_threadpool_t *sccp_threadpool_init(int threadsN)
 	tp_p->last_size_check = time(0);
 	tp_p->job_high_water_mark = 0;
 	tp_p->last_resize = time(0);
+	tp_p->sccp_threadpool_shuttingdown = 0;
 
 	/* Initialise Condition */
 	ast_cond_init(&(tp_p->work), NULL);
@@ -111,22 +108,23 @@ void sccp_threadpool_grow(sccp_threadpool_t * tp_p, int amount)
 	sccp_threadpool_thread_t *tp_thread;
 	int t;
 
-	if (tp_p && !sccp_threadpool_shuttingdown) {
+	if (tp_p && !tp_p->sccp_threadpool_shuttingdown) {
 		for (t = 0; t < amount; t++) {
-			tp_thread = sccp_malloc(sizeof(sccp_threadpool_thread_t));
-			if (tp_thread == NULL) {
-				pbx_log(LOG_ERROR, "sccp_threadpool_init(): Could not allocate memory for thread\n");
+			if (!(tp_thread = sccp_calloc(sizeof *tp_thread, 1))) {
+                		pbx_log(LOG_ERROR, SS_Memory_Allocation_Error, "SCCP");
 				return;
 			}
-			tp_thread->die = 0;
+			tp_thread->die = FALSE;
 			tp_thread->tp_p = tp_p;
 
 			pthread_attr_init(&attr);
 			pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
 			pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL);
 			pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, NULL);
+			SCCP_LIST_LOCK(&(tp_p->threads));
 			SCCP_LIST_INSERT_HEAD(&(tp_p->threads), tp_thread, list);
-			sccp_log((DEBUGCAT_CORE)) (VERBOSE_PREFIX_3 "Create thread %d in pool \n", t);
+			SCCP_LIST_UNLOCK(&(tp_p->threads));
+			sccp_log((DEBUGCAT_CORE)) (VERBOSE_PREFIX_3 "Create thread %d(%p) in pool \n", t, (void *) tp_thread->thread);
 			pbx_pthread_create(&(tp_thread->thread), &attr, (void *) sccp_threadpool_thread_do, (void *) tp_thread);
 			ast_cond_broadcast(&(tp_p->work));
 		}
@@ -139,14 +137,22 @@ void sccp_threadpool_shrink(sccp_threadpool_t * tp_p, int amount)
 	sccp_threadpool_thread_t *tp_thread;
 	int t;
 
-	if (tp_p && !sccp_threadpool_shuttingdown) {
+	if (tp_p && !tp_p->sccp_threadpool_shuttingdown) {
 		for (t = 0; t < amount; t++) {
-			tp_thread = SCCP_LIST_FIRST(&(tp_p->threads));
-			tp_thread->die = 1;
-
-			sccp_log((DEBUGCAT_CORE)) (VERBOSE_PREFIX_3 "Sending die signal to thread %p in pool \n", (void *) tp_thread->thread);
-			// wake up all threads
-			ast_cond_broadcast(&(tp_p->work));
+			SCCP_LIST_LOCK(&(tp_p->threads));
+			SCCP_LIST_TRAVERSE(&(tp_p->threads), tp_thread, list) {
+				if (tp_thread->die == FALSE) {
+					tp_thread->die = TRUE;
+					break;
+				}
+			}
+			SCCP_LIST_UNLOCK(&(tp_p->threads));
+			
+			if (tp_thread) {
+				// wake up all threads
+				sccp_log((DEBUGCAT_CORE)) (VERBOSE_PREFIX_3 "Sending die signal to thread %p in pool \n", (void *) tp_thread->thread);
+				ast_cond_broadcast(&(tp_p->work));
+			}
 		}
 	}
 }
@@ -154,7 +160,7 @@ void sccp_threadpool_shrink(sccp_threadpool_t * tp_p, int amount)
 /* check threadpool size (increase/decrease if necessary) */
 static void sccp_threadpool_check_size(sccp_threadpool_t * tp_p)
 {
-	if (tp_p && !sccp_threadpool_shuttingdown) {
+	if (tp_p && !tp_p->sccp_threadpool_shuttingdown) {
 		sccp_log((DEBUGCAT_THPOOL)) (VERBOSE_PREFIX_3 "(sccp_threadpool_check_resize) in thread: %p\n", (void *) pthread_self());
 		SCCP_LIST_LOCK(&(tp_p->threads));
 		{
@@ -204,7 +210,7 @@ void sccp_threadpool_thread_do(void *p)
 
 	int jobs = 0, threads = 0;
 
-	sccp_log((DEBUGCAT_CORE)) (VERBOSE_PREFIX_3 "Starting Threadpool JobQueue\n");
+	sccp_log((DEBUGCAT_CORE)) (VERBOSE_PREFIX_3 "Starting Threadpool JobQueue:%p\n", thread);
 	while (1) {
 		pthread_testcancel();
 		jobs = SCCP_LIST_GETSIZE(&tp_p->jobs);
@@ -212,6 +218,7 @@ void sccp_threadpool_thread_do(void *p)
 
 		sccp_log((DEBUGCAT_THPOOL)) (VERBOSE_PREFIX_3 "(sccp_threadpool_thread_do) num_jobs: %d, thread: %p, num_threads: %d\n", jobs, thread, threads);
 
+		pthread_setcancelstate(PTHREAD_CANCEL_DISABLE, NULL);
 		SCCP_LIST_LOCK(&(tp_p->jobs));									/* LOCK */
 		while (SCCP_LIST_GETSIZE(&tp_p->jobs) == 0 && !tp_thread->die) {
 			sccp_log((DEBUGCAT_THPOOL)) (VERBOSE_PREFIX_3 "(sccp_threadpool_thread_do) Thread %p Waiting for New Work Condition\n", thread);
@@ -245,6 +252,7 @@ void sccp_threadpool_thread_do(void *p)
 				sccp_threadpool_check_size(tp_p);						/* Check Resizing */
 			}
 		}
+		pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL);
 	}
 	sccp_log((DEBUGCAT_CORE)) (VERBOSE_PREFIX_3 "JobQueue Exiting Thread...\n");
 	pthread_cleanup_pop(1);
@@ -255,12 +263,11 @@ void sccp_threadpool_thread_do(void *p)
 int sccp_threadpool_add_work(sccp_threadpool_t * tp_p, void *(*function_p) (void *), void *arg_p)
 {
 	// prevent new work while shutting down
-	if (!sccp_threadpool_shuttingdown) {
+	if (!tp_p->sccp_threadpool_shuttingdown) {
 		sccp_threadpool_job_t *newJob;
 
-		newJob = (sccp_threadpool_job_t *) sccp_malloc(sizeof(sccp_threadpool_job_t));			/* MALLOC job */
-		if (newJob == NULL) {
-			pbx_log(LOG_ERROR, "sccp_threadpool_add_work(): Could not allocate memory for new job\n");
+		if (!(newJob = sccp_calloc(sizeof *newJob, 1))) {
+        		pbx_log(LOG_ERROR, SS_Memory_Allocation_Error, "SCCP");
 			exit(1);
 		}
 
@@ -278,21 +285,24 @@ int sccp_threadpool_add_work(sccp_threadpool_t * tp_p, void *(*function_p) (void
 }
 
 /* Destroy the threadpool */
-void sccp_threadpool_destroy(sccp_threadpool_t * tp_p)
+boolean_t sccp_threadpool_destroy(sccp_threadpool_t * tp_p)
 {
+	if (!tp_p) {
+		return FALSE;
+	}
 	sccp_threadpool_thread_t *tp_thread = NULL;
 
 	sccp_log((DEBUGCAT_CORE)) (VERBOSE_PREFIX_3 "Destroying Threadpool %p with %d jobs\n", tp_p, SCCP_LIST_GETSIZE(&tp_p->jobs));
 
 	// After this point, no new jobs can be added
 	SCCP_LIST_LOCK(&(tp_p->jobs));
-	sccp_threadpool_shuttingdown = 1;
+	tp_p->sccp_threadpool_shuttingdown = 1;
 	SCCP_LIST_UNLOCK(&(tp_p->jobs));
 
 	// shutdown is a kind of work too
 	SCCP_LIST_LOCK(&(tp_p->threads));
 	SCCP_LIST_TRAVERSE(&(tp_p->threads), tp_thread, list) {
-		tp_thread->die = 1;
+		tp_thread->die = TRUE;
 		ast_cond_broadcast(&(tp_p->work));
 	}
 	SCCP_LIST_UNLOCK(&(tp_p->threads));
@@ -334,8 +344,10 @@ void sccp_threadpool_destroy(sccp_threadpool_t * tp_p)
 	ast_cond_destroy(&(tp_p->exit));									/* Remove Condition */
 	SCCP_LIST_HEAD_DESTROY(&(tp_p->jobs));
 	SCCP_LIST_HEAD_DESTROY(&(tp_p->threads));
-	sccp_free(tp_p);											/* DEALLOC thread pool */
+	sccp_free(tp_p);
+	tp_p = NULL;												/* DEALLOC thread pool */
 	sccp_log((DEBUGCAT_CORE)) (VERBOSE_PREFIX_3 "Threadpool Ended\n");
+	return TRUE;
 }
 
 int sccp_threadpool_thread_count(sccp_threadpool_t * tp_p)
@@ -348,16 +360,16 @@ int sccp_threadpool_thread_count(sccp_threadpool_t * tp_p)
 /* Add job to queue */
 void sccp_threadpool_jobqueue_add(sccp_threadpool_t * tp_p, sccp_threadpool_job_t * newjob_p)
 {
-	if (!tp_p) {
-		pbx_log(LOG_ERROR, "(sccp_threadpool_jobqueue_add) no tp_p\n");
+	if (!tp_p || !newjob_p) {
+		pbx_log(LOG_ERROR, "(sccp_threadpool_jobqueue_add) no tp_p or no work pointer\n");
 		sccp_free(newjob_p);
 		return;
 	}
 
 	sccp_log((DEBUGCAT_THPOOL)) (VERBOSE_PREFIX_3 "(sccp_threadpool_jobqueue_add) tp_p: %p, jobCount: %d\n", tp_p, SCCP_LIST_GETSIZE(&tp_p->jobs));
 	SCCP_LIST_LOCK(&(tp_p->jobs));
-	if (sccp_threadpool_shuttingdown) {
-		pbx_log(LOG_ERROR, "(sccp_threadpool_jobqueue_add) shutting down\n");
+	if (tp_p->sccp_threadpool_shuttingdown) {
+		pbx_log(LOG_ERROR, "(sccp_threadpool_jobqueue_add) shutting down. skipping work\n");
 		SCCP_LIST_UNLOCK(&(tp_p->jobs));
 		sccp_free(newjob_p);
 		return;
@@ -365,7 +377,7 @@ void sccp_threadpool_jobqueue_add(sccp_threadpool_t * tp_p, sccp_threadpool_job_
 	SCCP_LIST_INSERT_TAIL(&(tp_p->jobs), newjob_p, list);
 	SCCP_LIST_UNLOCK(&(tp_p->jobs));
 
-	if (SCCP_LIST_GETSIZE(&tp_p->jobs) > tp_p->job_high_water_mark) {
+	if ((int)SCCP_LIST_GETSIZE(&tp_p->jobs) > tp_p->job_high_water_mark) {
 		tp_p->job_high_water_mark = SCCP_LIST_GETSIZE(&tp_p->jobs);
 	}
 	ast_cond_signal(&(tp_p->work));
@@ -377,4 +389,114 @@ int sccp_threadpool_jobqueue_count(sccp_threadpool_t * tp_p)
 	return SCCP_LIST_GETSIZE(&tp_p->jobs);
 }
 
+
+#if CS_TEST_FRAMEWORK
+#include <asterisk/test.h>
+#define NUM_WORK 50
+#define test_category "/channels/chan_sccp/threadpool/"
+static void *sccp_cli_threadpool_test_thread(void *data)
+{
+	int loop;
+	int num_loops=rand() % 10000;
+	for (loop = 0; loop < num_loops; loop++) {
+		usleep(1);
+	}
+	return 0;
+}
+
+AST_TEST_DEFINE(sccp_threadpool_create_destroy)
+{
+	switch(cmd) {
+		case TEST_INIT:
+			info->name = "create_destroy";
+			info->category = test_category;
+			info->summary = "chan-sccp-b threadpool creation/destruction";
+			info->description = "chan-sccp-b threadpool creation/destruction";
+			return AST_TEST_NOT_RUN;
+	        case TEST_EXECUTE:
+	        	break;
+	}
+	sccp_threadpool_t *test_threadpool = NULL;
+
+	pbx_test_status_update(test, "Destroy non-existing threadpool: %p\n", test_threadpool);
+	sccp_threadpool_destroy(test_threadpool);
+	pbx_test_validate(test, sccp_threadpool_destroy(test_threadpool) == FALSE);
+
+	pbx_test_status_update(test, "Create threadpool\n");
+	test_threadpool = sccp_threadpool_init(THREADPOOL_MIN_SIZE);
+	pbx_test_validate(test, NULL != test_threadpool);
+
+	int current_size = sccp_threadpool_thread_count(test_threadpool);
+	pbx_test_status_update(test, "Grow threadpool by 3, current %d\n", current_size);
+	sccp_threadpool_grow(test_threadpool, 3);
+	pbx_test_validate(test, sccp_threadpool_thread_count(test_threadpool) == current_size + 3); 
+
+	current_size = sccp_threadpool_thread_count(test_threadpool);
+	pbx_test_status_update(test, "Shrink threadpool by 2, current %d\n", current_size);
+	sccp_threadpool_shrink(test_threadpool, 2);
+	sleep(1);
+	pbx_test_validate(test, sccp_threadpool_thread_count(test_threadpool) == current_size - 2); 
+
+	pbx_test_status_update(test, "Destroy threadpool: %p\n", test_threadpool);
+	pbx_test_validate(test, sccp_threadpool_destroy(test_threadpool) == TRUE);
+
+	return AST_TEST_PASS;
+}
+
+AST_TEST_DEFINE(sccp_threadpool_work)
+{
+	switch(cmd) {
+		case TEST_INIT:
+			info->name = "addwork";
+			info->category = test_category;
+			info->summary = "chan-sccp-b threadpool work";
+			info->description = "chan-sccp-b threadpool work test";
+			return AST_TEST_NOT_RUN;
+	        case TEST_EXECUTE:
+	        	break;
+	}
+	sccp_threadpool_t *test_threadpool = NULL;
+	
+	pbx_test_status_update(test, "Create Test threadpool\n");
+	test_threadpool = sccp_threadpool_init(THREADPOOL_MIN_SIZE);
+	pbx_test_validate(test, NULL != test_threadpool);
+
+	int current_size = sccp_threadpool_thread_count(test_threadpool);
+	pbx_test_status_update(test, "Grow threadpool by 3\n");
+	sccp_threadpool_grow(test_threadpool, 3);
+	pbx_test_validate(test, sccp_threadpool_thread_count(test_threadpool) == current_size + 3); 
+	
+	if (test_threadpool) {
+		pbx_test_status_update(test, "Adding work to Test threadpool\n");
+		int work, loopcount=0;
+		for (work = 0; work < NUM_WORK; work++) {
+			pbx_test_validate(test, sccp_threadpool_add_work(test_threadpool, (void *) sccp_cli_threadpool_test_thread, test) > 0);
+		}
+
+		pbx_test_status_update(test, "Waiting for work to finishg in Test threadpool\n");
+		while (sccp_threadpool_jobqueue_count(test_threadpool) > 0 && loopcount++ < 20) {
+			pbx_test_status_update(test, "Job Queue: %d, Threads: %d\n", sccp_threadpool_jobqueue_count(test_threadpool), sccp_threadpool_thread_count(test_threadpool));
+			sleep(1);
+		}
+		pbx_test_validate(test, sccp_threadpool_jobqueue_count(test_threadpool) == 0); 
+
+		pbx_test_status_update(test, "Destroy Test threadpool\n");
+		sccp_threadpool_destroy(test_threadpool);
+	}
+	return AST_TEST_PASS;
+}
+
+static void __attribute__((constructor)) sccp_register_tests(void)
+{
+        AST_TEST_REGISTER(sccp_threadpool_create_destroy);
+        AST_TEST_REGISTER(sccp_threadpool_work);
+}
+
+static void __attribute__((destructor)) sccp_unregister_tests(void)
+{
+        AST_TEST_UNREGISTER(sccp_threadpool_create_destroy);
+        AST_TEST_UNREGISTER(sccp_threadpool_work);
+}
+#endif
+
 // kate: indent-width 8; replace-tabs off; indent-mode cstyle; auto-insert-doxygen on; line-numbers on; tab-indents on; keep-extra-spaces off; auto-brackets off;
