@@ -1,28 +1,23 @@
 /*!
- * \file	sccp_threadpool.h
+ * \file        sccp_threadpool.h
  * \brief       SCCP Threadpool Header
- * \author	Diederik de Groot < ddegroot@users.sourceforge.net >
- * \note	This program is free software and may be modified and distributed under the terms of the GNU Public License.
- *		See the LICENSE file at the top of the source tree.
- * \note	Based on the work of Johan Hanssen Seferidis
- *		Library providing a threading pool where you can add work. 
- * \since	2009-01-16
- *
- * $Date: 2010-11-17 12:03:44 +0100 (Wed, 17 Nov 2010) $
- * $Revision: 2130 $  
+ * \author      Diederik de Groot < ddegroot@users.sourceforge.net >
+ * \note        This program is free software and may be modified and distributed under the terms of the GNU Public License.
+ *              See the LICENSE file at the top of the source tree.
+ * \note        Based on the work of Johan Hanssen Seferidis
+ *              Library providing a threading pool where you can add work. 
+ * \since       2009-01-16
  */
-#ifndef SCCP_THREADPOOL_H_
-#define SCCP_THREADPOOL_H_
-
-#include <config.h>
+#pragma once
+//#include <config.h>
 //#include "common.h"
 
 /* Description:         Library providing a threading pool where you can add work on the fly. The number
- *		        of threads in the pool is adjustable when creating the pool. In most cases
- *		        this should equal the number of threads supported by your cpu.
+ *                      of threads in the pool is adjustable when creating the pool. In most cases
+ *                      this should equal the number of threads supported by your cpu.
  *          
- *		        In this header file a detailed overview of the functions and the threadpool logical
- *		        scheme is present in case tweaking of the pool is needed. 
+ *                      In this header file a detailed overview of the functions and the threadpool logical
+ *                      scheme is present in case tweaking of the pool is needed. 
  * */
 
 /* 
@@ -36,18 +31,18 @@
  * 
  * */
 
-/*		         _______________________________________________________        
- *		        /                                                       \
- *		        |   JOB QUEUE        | job1 | job2 | job3 | job4 | ..   |
- *		        |                                                       |
- *		        |   threadpool      | thread1 | thread2 | ..            |
- *		        \_______________________________________________________/
+/*                       _______________________________________________________        
+ *                      /                                                       \
+ *                      |   JOB QUEUE        | job1 | job2 | job3 | job4 | ..   |
+ *                      |                                                       |
+ *                      |   threadpool      | thread1 | thread2 | ..            |
+ *                      \_______________________________________________________/
  *      
  * Description:         Jobs are added to the job queue. Once a thread in the pool
- *		        is idle, it is assigned with the first job from the queue(and
- *		        erased from the queue). It's each thread's job to read from 
- *		        the queue serially(using lock) and executing each job
- *		        until the queue is empty.
+ *                      is idle, it is assigned with the first job from the queue(and
+ *                      erased from the queue). It's each thread's job to read from 
+ *                      the queue serially(using lock) and executing each job
+ *                      until the queue is empty.
  * 
  */
 /* ================================= STRUCTURES ================================================ */
@@ -114,7 +109,7 @@ int sccp_threadpool_add_work(sccp_threadpool_t * tp_p, void *(*function_p) (void
  * 
  * \param tp_p threadpool a pointer to the threadpool structure you want to destroy
  */
-void sccp_threadpool_destroy(sccp_threadpool_t * tp_p);
+boolean_t sccp_threadpool_destroy(sccp_threadpool_t * tp_p);
 
 /*!
  * \brief Return number of currently allocate threads in the threadpool
@@ -151,4 +146,4 @@ void sccp_threadpool_jobqueue_add(sccp_threadpool_t * tp_p, sccp_threadpool_job_
  */
 int sccp_threadpool_jobqueue_count(sccp_threadpool_t * tp_p);
 
-#endif
+// kate: indent-width 8; replace-tabs off; indent-mode cstyle; auto-insert-doxygen on; line-numbers on; tab-indents on; keep-extra-spaces off; auto-brackets off;
