@@ -7,17 +7,15 @@
  *              Modified by Jan Czmok and Julien Goodwin
  * \note        This program is free software and may be modified and distributed under the terms of the GNU Public License.
  *              See the LICENSE file at the top of the source tree.
- *
- * $Date: 2015-04-19 20:49:40 +0200 (Sun, 19 Apr 2015) $
- * $Revision: 6026 $
  */
-#ifndef __SCCP_UTILS_H
-#define __SCCP_UTILS_H
+#pragma once
+#include "config.h"
 
 #ifdef strdupa
 #define sccp_strdupa strdupa
 #else
-#define sccp_strdupa(s)						\
+#ifndef pbx_strdupa
+#define pbx_strdupa(s)						\
 	(__extension__						\
 	({							\
 		const char *__old = (s);			\
@@ -26,16 +24,19 @@
 		memcpy (__new, __old, __len);			\
 		__new;						\
 	}))
+#endif
 #define strdupa sccp_strdupa
 #endif
 
 void sccp_dump_packet(unsigned char *messagebuffer, int len);
-void sccp_dump_msg(sccp_msg_t * msg);
+void sccp_dump_msg(const sccp_msg_t * const msg);
 int sccp_addons_taps(sccp_device_t * d);
 void sccp_addons_clear(sccp_device_t * d);
 void sccp_safe_sleep(int ms);
 void sccp_pbx_setcallstate(sccp_channel_t * channel, int state);
+#if UNUSEDCODE // 2015-11-01
 void sccp_dev_dbclean(void);
+#endif
 
 #define _ARR2STR(arrayname, lookup_var, lookup_val, return_var) \
         ({ \
@@ -73,20 +74,20 @@ void sccp_dev_dbclean(void);
 
 // SCCP Lookups
 gcc_inline const char *msgtype2str(sccp_mid_t type);
-gcc_inline size_t msgtype2size(sccp_mid_t type);
 gcc_inline const char *pbxsccp_devicestate2str(uint32_t value);
-gcc_inline const char *extensionstatus2str(uint32_t value);
-
 // SKINNY Lookups
 gcc_inline const char *label2str(uint16_t value);
 gcc_inline const char *codec2str(skinny_codec_t value);
-gcc_inline int codec2payload(skinny_codec_t value);
-gcc_inline const char *codec2key(skinny_codec_t value);
 gcc_inline const char *codec2name(skinny_codec_t value);
 gcc_inline const char *featureType2str(sccp_feature_type_t value);
 gcc_inline const char *debugcat_keys(void);
-gcc_inline uint32_t debugcat2int(const char *str);
 gcc_inline uint32_t labelstr2int(const char *str);
+#if UNUSEDCODE // 2015-11-01
+gcc_inline const char *extensionstatus2str(uint32_t value);
+gcc_inline int codec2payload(skinny_codec_t value);
+gcc_inline const char *codec2key(skinny_codec_t value);
+gcc_inline uint32_t debugcat2int(const char *str);
+#endif
 
 char *sccp_multiple_codecs2str(char *buf, size_t size, const skinny_codec_t * codecs, const int length);
 int sccp_parse_allow_disallow(skinny_codec_t * sccp_codecs, const char *list, int allowing);
@@ -108,26 +109,26 @@ unsigned int sccp_app_separate_args(char *buf, char delim, char **array, int arr
 #endif
 
 void sccp_util_featureStorageBackend(const sccp_event_t * e);
+#if 0 /* unused */
 int sccp_softkeyindex_find_label(sccp_device_t * d, unsigned int keymode, unsigned int softkey);
+#endif
 
 //sccp_device_t *sccp_device_find_byipaddress(unsigned long s_addr);
-sccp_device_t *sccp_device_find_byipaddress(struct sockaddr_storage *sin);
+//sccp_device_t *sccp_device_find_byipaddress(struct sockaddr_storage *sin);
 
 sccp_feature_type_t sccp_featureStr2featureID(const char *str);
 boolean_t sccp_util_matchSubscriptionId(const sccp_channel_t * channel, const char *SubscriptionIdNum);
-int32_t sccp_parse_debugline(char *arguments[], int startat, int argc, int32_t new_debug);
-char *sccp_get_debugcategories(int32_t debugvalue);
 sccp_msg_t *sccp_utils_buildLineStatDynamicMessage(uint32_t lineInstance, uint32_t type, const char *dirNum, const char *fqdn, const char *lineDisplayName);
 
-int socket_equals(struct sockaddr_storage *s0, struct sockaddr_storage *s1);
+gcc_inline boolean_t sccp_socket_equals(const struct sockaddr_storage * const s0, const struct sockaddr_storage *const s1);
 gcc_inline size_t sccp_strlen(const char *data);
 gcc_inline boolean_t sccp_strlen_zero(const char *data);
 gcc_inline boolean_t sccp_strequals(const char *data1, const char *data2);
 gcc_inline boolean_t sccp_strcaseequals(const char *data1, const char *data2);
 int sccp_strIsNumeric(const char *s);
 skinny_codec_t sccp_utils_findBestCodec(const skinny_codec_t ourPreferences[], int pLength, const skinny_codec_t ourCapabilities[], int length1, const skinny_codec_t remotePeerCapabilities[], int length2);
-void sccp_utils_reduceCodecSet(skinny_codec_t *baseCodecs[SKINNY_MAX_CAPABILITIES], const skinny_codec_t reduceByCodecs[]);
-void sccp_utils_combineCodecSets(skinny_codec_t **baseCodecs, const skinny_codec_t addCodecs[]);
+void sccp_utils_reduceCodecSet(skinny_codec_t base[SKINNY_MAX_CAPABILITIES], const skinny_codec_t reduceByCodecs[SKINNY_MAX_CAPABILITIES]);
+void sccp_utils_combineCodecSets(skinny_codec_t base[SKINNY_MAX_CAPABILITIES], const skinny_codec_t addCodecs[SKINNY_MAX_CAPABILITIES]);
 
 void sccp_free_ha(struct sccp_ha *ha);
 int sccp_apply_ha(const struct sccp_ha *ha, const struct sockaddr_storage *addr);
@@ -142,11 +143,8 @@ int sccp_strversioncmp(const char *s1, const char *s2);
 char *sccp_dec2binstr(char *buf, size_t size, int value);
 gcc_inline void sccp_copy_string(char *dst, const char *src, size_t size);
 char *sccp_trimwhitespace(char *str);
-gcc_inline boolean_t sccp_utils_convUtf8toLatin1(const char *utf8str, char *buf, size_t len);
-#if CS_TEST_FRAMEWORK
-void sccp_utils_register_tests(void);
-void sccp_utils_unregister_tests(void);
-#endif
-
+gcc_inline boolean_t sccp_utils_convUtf8toLatin1(ICONV_CONST char *utf8str, char *buf, size_t len);
+#ifdef DEBUG
+void sccp_do_backtrace(void);
 #endif
 // kate: indent-width 8; replace-tabs off; indent-mode cstyle; auto-insert-doxygen on; line-numbers on; tab-indents on; keep-extra-spaces off; auto-brackets off;
